# flake8: noqa
#!/usr/bin/env python
# -*- coding: utf-8 -*-
import csv
from import_products import init_n_connect, add_elem

# import pdb


__version__ = "2.0.9"

MYDICT = {
    "default_code": 0,
    "lst_price": 1,
    "name": 2,
    "partner_ref": 6,
    "description": 11,
}


if __name__ == "__main__":
    oerp, uid, ctx = init_n_connect(flavour="gb")
    print("Import data %s on DB %s" % (ctx["flavour"], ctx["db_name"]))
    dialect = "excel"
    with open(ctx["csv_fn"], "rbU") as f:
        hdr = False
        reader = csv.reader(f, dialect=dialect)
        for row in reader:
            if not hdr:
                hdr = True
                continue
            # print row[1]
            add_elem(row, ctx, MYDICT)





# -*- coding: utf-8 -*-
#
# Configuration file for the Sphinx documentation on 2019-11-11 07:48:10
#
# This file does only contain a selection of the most common options. For a
# full list see the documentation:
# http://www.sphinx-doc.org/en/master/config

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
# import os
# import sys
# sys.path.insert(0, os.path.abspath("."))
# import sphinx_rtd_theme


# -- Project information -----------------------------------------------------

project = "clodoo"
copyright = "2019-23, SHS-AV s.r.l."
author = "Antonio Maria Vigliotti"

# The short X.Y version
version = "2.0.9"
# The full version, including alpha/beta/rc tags
release = "2.0.9"


# -- General configuration ---------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
#
# needs_sphinx = "1.0"

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named "sphinx.ext.*") or your custom
# ones.
extensions = [
    "sphinx_rtd_theme",
    "sphinx.ext.todo",
    "sphinx.ext.githubpages",
    "sphinx.ext.autodoc",
    "sphinx.ext.autosummary",
]


# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
#
# source_suffix = [".rst", ".md"]
source_suffix = ".rst"

# The master toctree document.
master_doc = "index"

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set 'language' from the command line for these cases.
language = "en"

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = [
    "_build",
    "Thumbs.db",
    ".DS_Store",
    "description*",
    "descrizione*",
    "features*",
    "oca_diff*",
    "certifications*",
    "prerequisites*",
    "installation*",
    "configuration*",
    "upgrade*",
    "support*",
    "usage*",
    "maintenance*",
    "troubleshooting*",
    "known_issues*",
    "proposals_for_enhancement*",
    "history*",
    "faq*",
    "sponsor*",
    "copyright_notes*",
    "available_addons*",
    "contact_us*",
    "__init__*",
    "name*",
    "summary*",
    "sommario*",
    "maturity*",
    "module_name*",
    "repos_name*",
    "today*",
    "authors*",
    "contributors*",
    "translators*",
    "acknowledges*",
    "MAINPAGE.rst",
]

# The name of the Pygments (syntax highlighting) style to use.
# pygments_style = None
pygments_style = "sphinx"


# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
# on_rtd is whether we are on readthedocs.org,
# this line of code grabbed from docs.readthedocs.org
# html_theme = "sphinx_rtd_theme"
# html_theme = "python_docs_theme"
html_theme = "nature"

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
#
html_theme_options = {
    # "canonical_url": "",
    # "analytics_id": "UA-XXXXXXX-1",
    # "logo_only": False,
    # "display_version": True,
    # "prev_next_buttons_location": "bottom",
    # "style_external_links": False,
    # "vcs_pageview_mode": "",
    # "style_nav_header_background": "white",
    # Toc options
    # "collapse_navigation": True,
    # "sticky_navigation": True,
    # "navigation_depth": 4,
    # "includehidden": True,
    # "titles_only": False,
}

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named 'default.css' will overwrite the builtin 'default.css'.
# html_static_path = ["_static"]

# Custom sidebar templates, must be a dictionary that maps document names
# to template names.
#
# The default sidebars (for documents that don"t match any pattern) are
# defined by theme itself.  Builtin themes are using these templates by
# default: ["localtoc.html", "relations.html", "sourcelink.html",
# "searchbox.html"].
#
# html_sidebars = {}
html_logo = "logozero_180x46.png"
#
# autodoc_default_flags = ["members"]
autosummary_generate = True
autodoc_default_options = {
    # "members": "Z0test, Z0testOdoo",
    "undoc-members": True,
    # "exclude-members": "Macro, SanityTest",
}





# -*- coding: utf-8 -*-
#
# Copyright SHS-AV s.r.l. <http://www.zeroincombenze.org>)
#
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl).
#
#    All Rights Reserved
#
"""
    Clodoo Regression Test Suite
"""

# import pdb
import os
import sys
import re

from zerobug import z0test

try:
    from clodoo.clodoolib import build_odoo_param
except BaseException:
    from clodoolib import build_odoo_param


__version__ = "2.0.9"


MODULE_ID = "clodoo"
VERSIONS_TO_TEST = ["14.0", "13.0", "12.0", "11.0", "10.0", "9.0", "8.0", "7.0", "6.1"]
MAJVERS_TO_TEST = ["14", "13", "12", "11", "10", "9", "8", "7", "6"]
# VERSIONS_TO_TEST = ('7.0',)
TEST_FAILED = 1
TEST_SUCCESS = 0


def version():
    return __version__


class RegressionTest:
    def __init__(self, testlib):
        self.Z = testlib

    def test_01(self, z0ctx):
        TRES = {
            "6": "6.1",
            "7": "7.0",
            "8": "8.0",
            "9": "9.0",
            "10": "10.0",
            "11": "11.0",
            "12": "12.0",
            "13": "13.0",
            "14": "14.0",
        }
        for ver in MAJVERS_TO_TEST:
            res = build_odoo_param("FULLVER", odoo_vid=ver)
            sts = self.Z.test_result(
                z0ctx, "Full version %s [python]" % ver, TRES[ver], res
            )
            if sts:
                break

            res = build_odoo_param("FULLVER", odoo_vid="v%s" % ver)
            sts = self.Z.test_result(
                z0ctx, "Full version v%s [python]" % ver, TRES[ver], res
            )
            if sts:
                break

            res = build_odoo_param("FULLVER", odoo_vid=TRES[ver])
            sts = self.Z.test_result(
                z0ctx, "Full version %s [python]" % TRES[ver], TRES[ver], res
            )
            if sts:
                break

            res = build_odoo_param("FULLVER", odoo_vid="v%s" % TRES[ver])
            sts = self.Z.test_result(
                z0ctx, "Full version v%s [python]" % TRES[ver], TRES[ver], res
            )
            if sts:
                break

            res = build_odoo_param("FULLVER", odoo_vid="V%s" % TRES[ver])
            sts = self.Z.test_result(
                z0ctx, "Full version V%s [python]" % TRES[ver], TRES[ver], res
            )
            if sts:
                break

            res = build_odoo_param("FULLVER", odoo_vid="VENV-%s" % TRES[ver])
            sts = self.Z.test_result(
                z0ctx, "Full version VENV-%s [python]" % TRES[ver], TRES[ver], res
            )
            if sts:
                break

            sts = self.Z.test_result(
                z0ctx, "Full version odoo-%s [python]" % TRES[ver], TRES[ver], res
            )
            if sts:
                break

            sts = self.Z.test_result(
                z0ctx, "Full version ODOO-%s [python]" % TRES[ver], TRES[ver], res
            )
            if sts:
                break
        return sts

    def test_02(self, z0ctx):
        TRES = {
            "6.1": 6,
            "7.0": 7,
            "8.0": 8,
            "9.0": 9,
            "10.0": 10,
            "11.0": 11,
            "12.0": 12,
            "13.0": 13,
            "14.0": 14,
        }
        for ver in VERSIONS_TO_TEST:
            res = build_odoo_param("MAJVER", odoo_vid=ver)
            sts = self.Z.test_result(
                z0ctx, "major version %s [python]" % ver, TRES[ver], res
            )
            if sts:
                break

            w = "V%s" % ver
            res = build_odoo_param("MAJVER", odoo_vid=w)
            sts = self.Z.test_result(
                z0ctx, "major version %s [python]" % w, TRES[ver], res
            )
            if sts:
                break

            w = "OCB-%s" % ver
            res = build_odoo_param("MAJVER", odoo_vid=w)
            sts = self.Z.test_result(
                z0ctx, "major version %s [python]" % w, TRES[ver], res
            )
            if sts:
                break
        return sts

    def test_03(self, z0ctx):
        TRES = {
            "6": "/etc/odoo/openerp-server.conf",
            "7": "/etc/odoo/odoo-server.conf",
            "8": "/etc/odoo/odoo-server.conf",
            "9": "/etc/odoo/odoo-server.conf",
            "10": "/etc/odoo/odoo.conf",
            "11": "/etc/odoo/odoo.conf",
            "12": "/etc/odoo/odoo.conf",
            "13": "/etc/odoo/odoo.conf",
            "14": "/etc/odoo/odoo.conf",
            "v7": "/etc/odoo/openerp-server.conf",
        }
        for ver in MAJVERS_TO_TEST:
            res = build_odoo_param("CONFN", odoo_vid=ver)
            sts = self.Z.test_result(
                z0ctx, "config unique filename %s [python]" % ver, TRES[ver], res
            )
            if sts:
                break

            w = "VENV-%s" % ver
            res = build_odoo_param("CONFN", odoo_vid=w)
            sts = self.Z.test_result(
                z0ctx, "config unique filename %s [python]" % w, TRES[ver], res
            )
            if sts:
                break

        for ver in MAJVERS_TO_TEST + ["v7"]:
            if ver == "v7":
                w = ver
            elif ver == "6":
                w = "%s.1" % ver
            else:
                w = "%s.0" % ver
            res = build_odoo_param("CONFN", odoo_vid=w)
            sts = self.Z.test_result(
                z0ctx, "config unique filename %s [python]" % w, TRES[ver], res
            )
            if sts:
                break

        return sts

    def test_04(self, z0ctx):
        TRES = {
            "6": "/etc/odoo/odoo6-server.conf",
            "7": "/etc/odoo/odoo7-server.conf",
            "8": "/etc/odoo/odoo8-server.conf",
            "9": "/etc/odoo/odoo9-server.conf",
            "10": "/etc/odoo/odoo10.conf",
            "11": "/etc/odoo/odoo11.conf",
            "12": "/etc/odoo/odoo12.conf",
            "13": "/etc/odoo/odoo13.conf",
            "14": "/etc/odoo/odoo14.conf",
            "v7": "/etc/odoo/openerp-server.conf",
        }
        for ver in MAJVERS_TO_TEST:
            if ver == "6":
                continue
            res = build_odoo_param("CONFN", odoo_vid=ver, multi=True)
            sts = self.Z.test_result(
                z0ctx, "config multi filename %s [python]" % ver, TRES[ver], res
            )
            if sts:
                break

            w = "VENV-%s" % ver
            res = build_odoo_param("CONFN", odoo_vid=w, multi=True)
            sts = self.Z.test_result(
                z0ctx, "config multi filename %s [python]" % w, TRES[ver], res
            )
            if sts:
                break

        for ver in MAJVERS_TO_TEST + ["v7"]:
            if ver == "v7":
                w = ver
            elif ver == "6":
                # w = "%s.1" % ver
                continue
            else:
                w = "%s.0" % ver
            res = build_odoo_param("CONFN", odoo_vid=w, multi=True)
            sts = self.Z.test_result(
                z0ctx, "config multi filename %s [python]" % w, TRES[ver], res
            )
            if sts:
                break

        TRES = {
            "6": "/var/log/odoo/odoo6-server.log",
            "7": "/var/log/odoo/odoo7-server.log",
            "8": "/var/log/odoo/odoo8-server.log",
            "9": "/var/log/odoo/odoo9-server.log",
            "10": "/var/log/odoo/odoo10.log",
            "11": "/var/log/odoo/odoo11.log",
            "12": "/var/log/odoo/odoo12.log",
            "13": "/var/log/odoo/odoo13.log",
            "14": "/var/log/odoo/odoo14.log",
            "v7": "/var/log/odoo/openerp-server.log",
        }
        for ver in MAJVERS_TO_TEST + ["v7"]:
            if ver == "v7":
                w = ver
            elif ver == "6":
                # w = "%s.1" % ver
                continue
            else:
                w = "%s.0" % ver
            res = build_odoo_param("FLOG", odoo_vid=w, multi=True)
            sts = self.Z.test_result(
                z0ctx, "log filename %s [python]" % w, TRES[ver], res
            )
            if sts:
                break

        TRES = {
            "6": "/var/run/odoo/odoo6-server.pid",
            "7": "/var/run/odoo/odoo7-server.pid",
            "8": "/var/run/odoo/odoo8-server.pid",
            "9": "/var/run/odoo/odoo9-server.pid",
            "10": "/var/run/odoo/odoo10.pid",
            "11": "/var/run/odoo/odoo11.pid",
            "12": "/var/run/odoo/odoo12.pid",
            "13": "/var/run/odoo/odoo13.pid",
            "14": "/var/run/odoo/odoo14.pid",
            "v7": "/var/run/odoo/openerp-server.pid",
        }

        for ver in MAJVERS_TO_TEST + ["v7"]:
            if ver == "v7":
                w = ver
            elif ver == "6":
                # w = "%s.1" % ver
                continue
            else:
                w = "%s.0" % ver
            res = build_odoo_param("FPID", odoo_vid=w, multi=True)
            sts = self.Z.test_result(
                z0ctx, "pid filename %s [python]" % w, TRES[ver], res
            )
            if sts:
                break

        TRES = {
            "6": "/etc/init.d/odoo6-server",
            "7": "/etc/init.d/odoo7-server",
            "8": "/etc/init.d/odoo8-server",
            "9": "/etc/init.d/odoo9-server",
            "10": "/etc/init.d/odoo10",
            "11": "/etc/init.d/odoo11",
            "12": "/etc/init.d/odoo12",
            "13": "/etc/init.d/odoo13",
            "14": "/etc/init.d/odoo14",
            "v7": "/etc/init.d/openerp-server",
        }
        for ver in MAJVERS_TO_TEST + ["v7"]:
            if ver == "v7":
                w = ver
            elif ver == "6":
                # w = "%s.1" % ver
                continue
            else:
                w = "%s.0" % ver
            res = build_odoo_param("FULL_SVCNAME", odoo_vid=w, multi=True)
            sts = self.Z.test_result(
                z0ctx, "service script name %s [python]" % w, TRES[ver], res
            )
            if sts:
                break

        TRES = {
            "6": "odoo6-server",
            "7": "odoo7-server",
            "8": "odoo8-server",
            "9": "odoo9-server",
            "10": "odoo10",
            "11": "odoo11",
            "12": "odoo12",
            "13": "odoo13",
            "14": "odoo14",
            "v7": "openerp-server",
        }
        for ver in MAJVERS_TO_TEST + ["v7"]:
            if ver == "v7":
                w = ver
            elif ver == "6":
                # w = "%s.1" % ver
                continue
            else:
                w = "%s.0" % ver
            res = build_odoo_param("SVCNAME", odoo_vid=w, multi=True)
            sts = self.Z.test_result(
                z0ctx, "service name %s [python]" % w, TRES[ver], res
            )
            if sts:
                break

        # TRES = {
        #     "6": "/opt/odoo/6.1/server/openerp-server",
        #     "7": "/opt/odoo/7.0/openerp-server",
        #     "8": "/opt/odoo/8.0/openerp-server",
        #     "9": "/opt/odoo/9.0/openerp-server",
        #     "10": "/opt/odoo/10.0/odoo-bin",
        #     "11": "/opt/odoo/11.0/odoo-bin",
        #     "12": "/opt/odoo/12.0/odoo-bin",
        #     "13": "/opt/odoo/13.0/odoo-bin",
        #     "14": "/opt/odoo/14.0/odoo-bin",
        #     "v7": "/opt/odoo/v7/server/openerp-server",
        # }
        # for ver in MAJVERS_TO_TEST + ["v7"]:
        #     if ver == "v7":
        #         w = ver
        #     elif ver == "6":
        #         w = "%s.1" % ver
        #     else:
        #         w = "%s.0" % ver
        #     res = build_odoo_param("BIN", odoo_vid=w, multi=True)
        #     sts = self.Z.test_result(
        #         z0ctx, "run script name %s [python]" % w, TRES[ver], res
        #     )
        #     if sts:
        #         break

        # TRES = {
        #     "6.1": "/opt/odoo/VENV-6.1/odoo/server/openerp-server",
        #     "7.0": "/opt/odoo/VENV-7.0/odoo/openerp-server",
        #     "8.0": "/opt/odoo/VENV-8.0/odoo/openerp-server",
        #     "9.0": "/opt/odoo/VENV-9.0/odoo/openerp-server",
        #     "10.0": "/opt/odoo/VENV-10.0/odoo/odoo-bin",
        #     "11.0": "/opt/odoo/VENV-11.0/odoo/odoo-bin",
        #     "12.0": "/opt/odoo/VENV-12.0/odoo/odoo-bin",
        #     "13.0": "/opt/odoo/VENV-13.0/odoo/odoo-bin",
        #     "14.0": "/opt/odoo/VENV-14.0/odoo/odoo-bin",
        # }
        # for ver in VERSIONS_TO_TEST:
        #     w = "VENV-%s" % ver
        #     res = build_odoo_param("BIN", odoo_vid=w, multi=True)
        #     sts = self.Z.test_result(
        #         z0ctx, "run script name %s [python]" % w, TRES[ver], res
        #     )
        #     if sts:
        #         break

        TRES = {
            "6.1": "__openerp__.py",
            "7.0": "__openerp__.py",
            "8.0": "__openerp__.py",
            "9.0": "__openerp__.py",
            "10.0": "__manifest__.py",
            "11.0": "__manifest__.py",
            "12.0": "__manifest__.py",
            "13.0": "__manifest__.py",
            "14.0": "__manifest__.py",
            "v7": "__openerp__.py",
        }
        for ver in VERSIONS_TO_TEST + ["v7"]:
            res = build_odoo_param("MANIFEST", odoo_vid=ver, multi=True)
            sts = self.Z.test_result(
                z0ctx, "manifest %s [python]" % ver, TRES[ver], res
            )
            if sts:
                break

        TRES = {
            "6.1": 8166,
            "7.0": 8167,
            "8.0": 8168,
            "9.0": 8169,
            "10.0": 8170,
            "11.0": 8171,
            "12.0": 8172,
            "13.0": 8173,
            "14.0": 8174,
            "v7": 8069,
            "v8.0": 8069,
        }
        for ver in VERSIONS_TO_TEST + ["v7", "v8.0"]:
            res = build_odoo_param("RPCPORT", odoo_vid=ver, multi=True)
            sts = self.Z.test_result(z0ctx, "rpcport %s [python]" % ver, TRES[ver], res)
            if sts:
                break

        TRES = {
            "6.1": 8166,
            "7.0": 8167,
            "8.0": 8168,
            "9.0": 8169,
            "10.0": 8170,
            "11.0": 8171,
            "12.0": 8172,
            "13.0": 8173,
            "14.0": 8174,
        }
        for ver in VERSIONS_TO_TEST:
            res = build_odoo_param("RPCPORT", odoo_vid=ver, multi=True)
            sts = self.Z.test_result(z0ctx, "rpcport %s [python]" % ver, TRES[ver], res)
            if sts:
                break

        TRES = {
            "6.1": "odoo6",
            "7.0": "odoo7",
            "8.0": "odoo8",
            "9.0": "odoo9",
            "10.0": "odoo10",
            "11.0": "odoo11",
            "12.0": "odoo12",
            "13.0": "odoo13",
            "14.0": "odoo14",
            "v7": "odoo",
        }
        for ver in VERSIONS_TO_TEST + ["v7"]:
            res = build_odoo_param("USER", odoo_vid=ver, multi=True)
            sts = self.Z.test_result(z0ctx, "user %s [python]" % ver, TRES[ver], res)
            if sts:
                break

        TRES = {
            "6": "odoo6",
            "7": "odoo7",
            "8": "odoo8",
            "9": "odoo9",
            "10": "odoo10",
            "11": "odoo11",
            "12": "odoo12",
            "13": "odoo13",
            "14": "odoo14",
        }
        for ver in MAJVERS_TO_TEST:
            w = "VENV-%s" % ver
            res = build_odoo_param("USER", odoo_vid=w, multi=True)
            sts = self.Z.test_result(z0ctx, "user %s [python]" % w, TRES[ver], res)
            if sts:
                break

        TRES = {
            "6.1": "6.1",
            "7.0": "7.0",
            "8.0": "8.0",
            "9.0": "9.0",
            "10.0": "10.0",
            "11.0": "11.0",
            "12.0": "12.0",
            "13.0": "13.0",
            "14.0": "14.0",
        }
        for ver in VERSIONS_TO_TEST:
            w = "ODOO-%s" % ver
            res = build_odoo_param("FULLVER", odoo_vid=w, multi=True)
            sts = self.Z.test_result(z0ctx, "naming %s [python]" % w, TRES[ver], res)
            if sts:
                break

        return sts

    def __test_05(self, z0ctx):
        TRES = {
            "6.1": "/opt/odoo/6.1",
            "7.0": "/opt/odoo/7.0",
            "8.0": "/opt/odoo/8.0",
            "9.0": "/opt/odoo/9.0",
            "10.0": "/opt/odoo/10.0",
            "11.0": "/opt/odoo/11.0",
            "12.0": "/opt/odoo/12.0",
            "v7": "/opt/odoo/v7",
            "v8.0": "/opt/odoo/v8.0",
        }
        for ver in VERSIONS_TO_TEST + ["v7", "v8.0"]:
            res = build_odoo_param("ROOT", odoo_vid=ver, multi=True)
            sts = self.Z.test_result(z0ctx, "Root dir %s" % ver, TRES[ver], res)
            if sts:
                break

            res = build_odoo_param("ROOT", odoo_vid=ver, suppl="crm", multi=True)
            sts = self.Z.test_result(z0ctx, "Root dir %s/crm" % ver, TRES[ver], res)
            if sts:
                break

            res = build_odoo_param("HOME", odoo_vid=ver, suppl="OCB", multi=True)
            sts = self.Z.test_result(z0ctx, "Home dir %s/OCB" % ver, TRES[ver], res)
            if sts:
                break

        TRES = {
            "6.1": "/opt/odoo/VENV-6.1/odoo",
            "7.0": "/opt/odoo/VENV-7.0/odoo",
            "8.0": "/opt/odoo/VENV-8.0/odoo",
            "9.0": "/opt/odoo/VENV-9.0/odoo",
            "10.0": "/opt/odoo/VENV-10.0/odoo",
            "11.0": "/opt/odoo/VENV-11.0/odoo",
            "12.0": "/opt/odoo/VENV-12.0/odoo",
        }
        for ver in VERSIONS_TO_TEST:
            w = "VENV-%s" % ver
            res = build_odoo_param("ROOT", odoo_vid=w, suppl="crm", multi=True)
            sts = self.Z.test_result(z0ctx, "Root dir %s" % w, TRES[ver], res)
            if sts:
                break

        TRES = {
            "6.1": "/opt/odoo/6.1/openerp/filestore",
            "7.0": "/opt/odoo/7.0/openerp/filestore",
            "8.0": "/opt/odoo/.local/share/Odoo8",
            "9.0": "/opt/odoo/.local/share/Odoo9",
            "10.0": "/opt/odoo/.local/share/Odoo10",
            "11.0": "/opt/odoo/.local/share/Odoo11",
            "12.0": "/opt/odoo/.local/share/Odoo12",
            "VENV-10": "/opt/odoo/.local/share/Odoo",
            "v8.0": "/opt/odoo/.local/share/Odoo",
            "v7": "/opt/odoo/v7/openerp/filestore",
        }
        for ver in VERSIONS_TO_TEST + ["VENV-10", "v8.0", "v7"]:
            res = build_odoo_param("DDIR", odoo_vid=ver, multi=True)
            sts = self.Z.test_result(z0ctx, "Filestore  %s" % ver, TRES[ver], res)
            if sts:
                break

        TRES = {
            "6.1": "/opt/odoo/6.1/crm",
            "7.0": "/opt/odoo/7.0/crm",
            "8.0": "/opt/odoo/8.0/crm",
            "9.0": "/opt/odoo/9.0/crm",
            "10.0": "/opt/odoo/10.0/crm",
            "11.0": "/opt/odoo/11.0/crm",
            "12.0": "/opt/odoo/12.0/crm",
            "v7": "/opt/odoo/v7/crm",
            "v8.0": "/opt/odoo/v8.0/crm",
        }
        for ver in VERSIONS_TO_TEST + ["v7", "v8.0"]:
            res = build_odoo_param("HOME", odoo_vid=ver, suppl="crm", multi=True)
            sts = self.Z.test_result(z0ctx, "Home %s/crm" % ver, TRES[ver], res)
            if sts:
                break

        TRES = {
            "6.1": "/opt/odoo/VENV-6.1/odoo/crm",
            "7.0": "/opt/odoo/VENV-7.0/odoo/crm",
            "8.0": "/opt/odoo/VENV-8.0/odoo/crm",
            "9.0": "/opt/odoo/VENV-9.0/odoo/crm",
            "10.0": "/opt/odoo/VENV-10.0/odoo/crm",
            "11.0": "/opt/odoo/VENV-11.0/odoo/crm",
            "12.0": "/opt/odoo/VENV-12.0/odoo/crm",
        }
        for ver in VERSIONS_TO_TEST:
            w = "VENV-%s" % ver
            res = build_odoo_param("HOME", odoo_vid=w, suppl="crm", multi=True)
            sts = self.Z.test_result(z0ctx, "Home %s/crm" % w, TRES[ver], res)
            if sts:
                break

        return sts

    def __test_06(self, z0ctx):
        repos = "OCB"
        TRES = {
            "zero-git": "git@github.com:zeroincombenze/%s" % repos,
            "zero-http": "https://github.com/zeroincombenze/%s" % repos,
            "oca": "https://github.com/OCA/%s" % repos,
            "librerp": "https://github.com/iw3hxn/server",
        }
        if re.match("shs[a-z0-9]+", os.environ["HOSTNAME"]):
            TRES["zero"] = TRES["zero-git"]
        else:
            TRES["zero"] = TRES["zero-http"]
        for org in ("zero", "zero-git", "zero-http", "oca", "librerp"):
            for ver in VERSIONS_TO_TEST:
                RES = build_odoo_param("URL", odoo_vid=ver, multi=False)
                sts = self.Z.test_result(z0ctx, "Root dir %s" % ver, TRES[org], RES)
            if sts:
                break

            RES = build_odoo_param(
                "GIT_URL", odoo_vid=ver, suppl=repos, git_org=org, multi=False
            )
            sts = self.Z.test_result(
                z0ctx, "GIT_URL %s/%s %s" % (org, repos, ver), TRES[org], RES
            )
            if sts:
                break

            RES = build_odoo_param(
                "URL_BRANCH", odoo_vid=ver, suppl=repos, git_org=org, multi=False
            )
            sts = self.Z.test_result(
                z0ctx, "URL_BRANCH %s/%s %s" % (org, repos, ver), TRES[org], RES
            )
            if sts:
                break

        TRES = {
            "zero-git": "git@github.com:zeroincombenze",
            "zero-http": "https://github.com/zeroincombenze",
            "oca": "https://github.com/OCA",
            "librerp": "https://github.com/iw3hxn",
        }
        if re.match("shs[a-z0-9]+", os.environ["HOSTNAME"]):
            TRES["zero"] = TRES["zero-git"]
        else:
            TRES["zero"] = TRES["zero-http"]
        for org in ("zero", "zero-git", "zero-http", "oca", "librerp"):
            for ver in VERSIONS_TO_TEST:
                RES = build_odoo_param(
                    "GIT_ORG", odoo_vid=ver, suppl=repos, git_org=org, multi=False
                )
                sts = self.Z.test_result(
                    z0ctx, "GIT_ORG %s/%s %s" % (org, repos, ver), TRES[org], RES
                )
            if sts:
                break

        TRES = {
            "zero-git": '"https://github.com/OCA/%s' % repos,
            "zero-http": '"https://github.com/OCA/%s' % repos,
            "oca": '"https://github.com/OCA/%s' % repos,
            "librerp": "https://github.com/iw3hxn/%s" % repos,
        }
        for org in ("zero", "zero-git", "zero-http", "oca", "librerp"):
            for ver in VERSIONS_TO_TEST:
                RES = build_odoo_param(
                    "UPSTREAM", odoo_vid=ver, suppl=repos, git_org=org, multi=False
                )
                if ver == "6.1" or repos == "oca" or repos == "librerp":
                    sts = self.Z.test_result(
                        z0ctx, "UPSTREAM %s/%s %s" % (org, repos, ver), TRES[org], ""
                    )
                else:
                    sts = self.Z.test_result(
                        z0ctx, "UPSTREAM %s/%s %s" % (org, repos, ver), TRES[org], RES
                    )
            if sts:
                break

        return sts


#
# Run main if executed as a script
if __name__ == "__main__":
    exit(
        z0test.main_local(
            z0test.parseoptest(sys.argv[1:], version=version()), RegressionTest
        )
    )





# -*- coding: utf-8 -*-
#
# Copyright SHS-AV s.r.l. <http://www.zeroincombenze.org>)
#
# License LGPL-3.0 or later (http://www.gnu.org/licenses/agpl).
#
#    All Rights Reserved
#
"""
    Clodoo Regression Test Suite based on pytest
"""
# import pytest
from clodoo import transodoo

__version__ = "2.0.9"


class TestClass:
    def test_version_zerobug(self):
        ctx = {}
        ctx["mindroot"] = transodoo.read_stored_dict(ctx)
        model = "account.account.type"
        item = "report_type"
        src_ver = "6.1"
        tgt_ver = "10.0"
        ttype = "field"
        tnl_item = transodoo.translate_from_to(
            ctx, model, item, src_ver, tgt_ver, ttype=ttype
        )
        assert tnl_item == "type"





# flake8: noqa
# -*- coding: utf-8 -*-
# Import file library
"""
Import Partner from CSV

This program supplies functions to import records.
#
# Example of client software:
#
import csv
from import_records import (init_n_connect, set_header, add_item)
MYDICT_C = {
    'description': 'name',
}
MYDICT_S = {
}
def main():
    uid, ctx = init_n_connect()
    print("Import data %s on DB %s" % (ctx['flavour'],
                                       ctx['db_name']))
    ctr = 0
    with open(ctx['csv_fn'], 'rbU') as fd:
        hdr = False
        reader = csv.reader(fd, dialect='excel')
        for row in reader:
            if not hdr:
                set_header(ctx, row, 'model', MYDICT_C, MYDICT_S)
                hdr = True
                continue
            add_item(ctx, row)
            ctr += 1
    print('%d record added ...' % ctr)


Parameters as key of ctx:
    default_XXX: value for field XXX if not in csv file
    customers: import customers (from command line)
    company_name: company name to import data
    flavour: suffix to compose csv filename
    suppliers: import suppliers (from command line)
"""

from __future__ import print_function, unicode_literals
from __future__ import absolute_import

# from __future__ import division
from future import standard_library

# from builtins import *                                             # noqa: F403
import sys
import time
import csv

try:
    from python_plus import python_plus
except:
    import python_plus
try:
    from clodoo import clodoo
except:
    import clodoo
try:
    from z0lib import z0lib
except ImportError:
    import z0lib
standard_library.install_aliases()  # noqa: E402


__version__ = "2.0.9"


msg_time = time.time()


def msg_burst(text):
    global msg_time
    t = time.time() - msg_time
    if t > 3:
        print(text)
        msg_time = time.time()


def env_ref(ctx, xref, retxref_id=None):
    xrefs = xref.split(".")
    if len(xrefs) == 2:
        if xref.startswith("product.product_uom_"):
            xrefs[0] = "uom"
        model = "ir.model.data"
        ids = clodoo.searchL8(
            ctx, model, [("module", "=", xrefs[0]), ("name", "=", xrefs[1])]
        )
        if ids:
            if retxref_id:
                return ids[0]
            return clodoo.browseL8(ctx, model, ids[0]).res_id
    return False


def get_company_id(ctx):
    model = "res.company"
    company_id = ctx.get("default_company_id")
    if not company_id:
        company_id = env_ref(ctx, "base.main_company")
    if not company_id:
        company_name = ctx.get("company_name", "La % Azienda")
        ids = clodoo.searchL8(ctx, model, [("name", "ilike", company_name)])
        if not ids:
            ids = clodoo.searchL8(ctx, model, [("id", ">", 1)])
        if ids:
            company_id = ids[0]
        else:
            company_id = 1
    if "default_company_id" not in ctx:
        ctx["default_company_id"] = company_id
    return company_id


def get_many2one(ctx, value, model, parent_name=None, parent_value=None):
    if isinstance(value, basestring):
        rec_id = env_ref(ctx, value)
    elif isinstance(value, int):
        if model not in ctx["BIND"]:
            ctx["BIND"][model] = {}
        src_ctx = {}  # TODO
        if not ctx["BIND"][model].get(value):
            rec = clodoo.browseL8(src_ctx, ctx["model"], value)
            row = clodoo.extract_vals_from_rec(src_ctx, ctx["model"], rec, format="str")
        return ctx["BIND"][model].get(value)
    else:
        return None
    if not rec_id and value:
        if parent_name:
            domain = [(parent_name, "=", parent_value), ("name", "=", value)]
        else:
            domain = [("name", "=", value)]
        ids = clodoo.searchL8(ctx, model, domain)
        if not ids:
            if parent_name:
                domain = [(parent_name, "=", parent_value), ("name", "ilike", value)]
            else:
                domain = [("name", "ilike", value)]
            ids = clodoo.searchL8(ctx, model, domain)
        if ids:
            rec_id = ids[0]
    return rec_id


def get_country_id(ctx, value):
    if value:
        model = "res.country"
        rec_id = get_many2one(ctx, value, model)
        if "default_country_id" not in ctx:
            ctx["default_country_id"] = rec_id
    else:
        rec_id = ctx.get("default_country_id")
    return rec_id


def get_state_id(ctx, value, country_id=None):
    rec_id = False
    if value:
        if value.startswith("l10n_it_base.it_"):
            value = "base.state_it_%s" % value[:-2].lower()
        model = "res.country.state"
        parent_name = "country_id"
        if country_id:
            parent_value = country_id
        else:
            parent_value = get_country_id(ctx, False)
        rec_id = get_many2one(
            ctx, value, model, parent_name=parent_name, parent_value=parent_value
        )
    return rec_id


def get_uom_id(ctx, value):
    if value:
        model = "uom.uom"
        rec_id = get_many2one(ctx, value, model)
        if "uom_id" not in ctx:
            ctx["uom_id"] = rec_id
    else:
        rec_id = ctx.get("uom_id")
    return rec_id


def get_value(ctx, name, value):
    ttype = ctx["STRUCT"][name]["type"]
    if value is None:
        def_field_name = "default_%s" % name
        if def_field_name in ctx:
            value = ctx[def_field_name]
        elif not ctx["STRUCT"][name]["required"]:
            pass
        elif name == "company_id":
            value = get_company_id(ctx)
        elif name == "country_id":
            value = get_country_id(ctx, value)
        elif name == "state_id":
            value = get_state_id(ctx, value)
        elif name == "uom_id":
            value = get_uom_id(ctx, value)
    elif name in ("id", "valuation", "cost_method"):
        value = None
    elif ttype == "many2one":
        value = get_many2one(ctx, value, ctx["model"])
    elif ttype in ("one2many", "many2many"):
        value = get_many2one(ctx, value, ctx["model"])
        if value:
            value = [6, 0, [value]]
        else:
            value = None
    elif ttype == "boolean":
        value = eval(value)
    elif name in ctx["TNL"]:
        value = ctx["TNL"][name].get(value, value)
    return value


def write_log(msg):
    print(msg)
    with open("./import_records.log", "a") as fd:
        fd.write(b"%s\n" % python_plus._b(msg))


def add_item(ctx, row):
    def get_value_by_name(ctx, row, name, supplier=None):
        if supplier:
            if name not in ctx["HDR_SUPPLIER"]:
                return None
            return row[ctx["HDR_SUPPLIER"].index(name)]
        if name not in ctx["HDR_CUSTOMER"]:
            return None
        return row[ctx["HDR_CUSTOMER"].index(name)]

    python_plus.unicodes(row)
    msg_burst("Reading %s ..." % get_value_by_name(ctx, row, "name"))
    vals = {}
    is_customer = get_value_by_name(ctx, row, "customer")
    is_supplier = get_value_by_name(ctx, row, "is_supplier")
    supplier = is_supplier if not is_customer else False
    for ix, field in enumerate(ctx["STRUCT"]):
        if isinstance(row, (tuple, list)):
            value = get_value_by_name(ctx, row, field, supplier=supplier)
        else:
            value = row.get(field)
        vals[field] = get_value(ctx, field, value)
        ttype = ctx["STRUCT"][field]["type"]
        if ttype in ("one2many", "many2many", "many2one"):
            if vals[field]:
                ctx["BIND"][ctx["model"]][value] = vals[field]
            else:
                pass
    for item in list(vals.copy().keys()):
        if vals[item] is None:
            del vals[item]
    domain = []
    for key in ctx["SKEYS"]:
        if key in vals:
            domain.append((key, "=", vals[key]))
    rec_id = False
    if domain:
        ids = clodoo.searchL8(ctx, ctx["model"], domain)
        if 0 < len(ids) < 10:
            rec_id = ids[0]
    if rec_id:
        try:
            clodoo.writeL8(ctx, ctx["model"], rec_id, vals)
        except BaseException as e:
            write_log("Cannot write id %d (%s)" % (rec_id, e))
    else:
        try:
            clodoo.createL8(ctx, ctx["model"], vals)
        except BaseException as e:
            write_log("Cannot create %s (%s)" % (vals, e))


def set_header(ctx, row, model, customer_dict, purchase_dictionary, tnl, skeys):
    HDR_CUSTOMER = []
    HDR_SUPPLIER = []
    for field in row:
        if field.endswith("/id"):
            field = field[:-3]
        if field in customer_dict:
            HDR_CUSTOMER.append(customer_dict[field])
        else:
            HDR_CUSTOMER.append(field)
        if field in purchase_dictionary:
            HDR_SUPPLIER.append(purchase_dictionary[field])
        else:
            HDR_SUPPLIER.append(field)
    ctx["model"] = model
    ctx["HDR_CUSTOMER"] = HDR_CUSTOMER
    ctx["HDR_SUPPLIER"] = HDR_SUPPLIER
    ctx["TNL"] = tnl
    ctx["SKEYS"] = skeys
    ctx["STRUCT"] = clodoo.executeL8(ctx, ctx["model"], "fields_get")
    if "BIND" not in ctx:
        ctx["BIND"] = {}


def copy_db(ctx, src_ctx, MYDICT_C, MYDICT_S, TNL, skeys):
    set_header(ctx, [], ctx["model"], MYDICT_C, MYDICT_S, TNL, skeys)
    ctr = 0
    for rec in clodoo.browseL8(
        src_ctx, ctx["model"], clodoo.searchL8(src_ctx, ctx["model"], [], order="id")
    ):
        msg_burst("%d, %s" % (rec.name, rec.name))
        row = clodoo.extract_vals_from_rec(src_ctx, ctx["model"], rec, format="str")
        add_item(ctx, row)
        ctr += 1


def import_from_csv(ctx, MYDICT_C, MYDICT_S, TNL, skeys):
    ctr = 0
    with open(ctx["csv_fn"], "rbU") as fd:
        hdr = False
        reader = csv.reader(fd, dialect="excel")
        for row in reader:
            if not hdr:
                set_header(ctx, row, ctx["model"], MYDICT_C, MYDICT_S, TNL, skeys)
                hdr = True
                continue
            add_item(ctx, row)
            ctr += 1


def init_n_connect(flavour=None):
    title = "Importrecords %s" % flavour or ""
    parser = z0lib.parseoptargs(
        title, "© 2017-2021 by SHS-AV s.r.l.", version=__version__
    )
    parser.add_argument("-h")
    parser.add_argument(
        "-c",
        "--config",
        help="configuration command file",
        dest="conf_fn",
        metavar="file",
        default="./import_records.config",
    )
    parser.add_argument(
        "-d",
        "--dbname",
        help="DB name",
        dest="db_name",
        metavar="file",
        default="demo8",
    )
    parser.add_argument(
        "-e",
        "--customer",
        help="Import customers",
        action="store_true",
        dest="customers",
        default=False,
    )
    parser.add_argument("-m", "--model", help="Odoo model", dest="model")
    parser.add_argument(
        "-f",
        "--filename",
        help="Filename to import",
        dest="csv_fn",
        metavar="file",
        default=False,
    )
    parser.add_argument("-n")
    parser.add_argument("-q")
    parser.add_argument(
        "-s",
        "--supplier",
        help="Import suppliers",
        action="store_true",
        dest="suppliers",
        default=False,
    )
    parser.add_argument("-V")
    parser.add_argument("-v")
    parser.add_argument(
        "-w",
        "--src-config",
        help="Source DB configuration file",
        dest="src_conf_fn",
        metavar="file",
    )
    parser.add_argument(
        "-x",
        "--src-db_name",
        help="Source database name",
        dest="src_db_name",
        metavar="name",
        default="demo",
    )
    # Connect to DB
    ctx = parser.parseoptargs(sys.argv[1:], apply_conf=False)
    if not ctx.get("csv_fn") and (ctx.get("src_conf_fn") or ctx.get("src_db_name")):
        src_ctx = ctx.copy()
    else:
        src_uid = src_ctx = None
    ctx["flavour"] = flavour
    if not ctx["model"]:
        print("Missed model: use -m switch")
        exit(1)
    if not ctx["customers"] and not ctx["suppliers"]:
        ctx["customers"] = ctx["suppliers"] = True
    if not src_ctx and not ctx["csv_fn"]:
        if flavour:
            sfx = "_"
        else:
            sfx = ""
        ctx["csv_fn"] = "%s%s%s.csv" % (
            ctx["model"].replace(".", "_"),
            sfx,
            flavour or "",
        )
    uid, ctx = clodoo.oerp_set_env(confn=ctx["conf_fn"], db=ctx["db_name"], ctx=ctx)
    if src_ctx:
        src_uid, src_ctx = clodoo.oerp_set_env(
            confn=src_ctx["src_conf_fn"], db=src_ctx["src_db_name"], ctx=src_ctx
        )
    ctx["default_country_id"] = get_country_id(ctx, "Italia")
    ctx["default_is_company"] = True
    return uid, ctx, src_uid, src_ctx





# -*- coding: utf-8 -*-
# content of conftest.py

from subprocess import PIPE, Popen

import pytest

# from clodoo import transodoo


def pytest_report_header(config):
    return "project zerobug"


@pytest.fixture
def version_to_test():
    def get_version(cmd):
        res, err = Popen(cmd, stdin=PIPE, stdout=PIPE, stderr=PIPE).communicate()
        res = res or err
        return res.split()[0].split('=')[-1].strip().strip('"').strip("'")

    def _version_to_test(package, version, mode=None):
        """check for version of module/package
        @package: pypi package or external command
        @version: version to test
        @mode may be:
            '' = use package.version() - This is the default mode
            '.' = use grep to find 1st occurrence of __version__=
            '-V' = exec 'package -V'
            '-v' = exec 'package -v'
            '--version' = exec 'package --version'
        """
        if mode and mode.startswith('-'):
            assert version == get_version([package, mode])
        elif mode == '.':
            assert version == get_version(['grep', '__version__', package])
        else:
            assert package.version() == version

    return _version_to_test



# flake8: noqa
# -*- coding: utf-8 -*-
# Import file library
"""
Import Product from CSV

This program supplies functions to import products.
#
# Example of client software:
#
import csv
from import_products import (init_n_connect, add_elem)
MYDICT_C = {
    'name': 1,
}

def main():
    odoo, uid, ctx = init_n_connect(flavour='myname')
    ctx['IN_country_id'] = 'name'
    dialect = 'excel'
    with open(ctx['csv_fn'], 'rbU') as f:
        hdr = False
        reader = csv.reader(f, dialect=dialect)
        for row in reader:
            if not hdr:
                hdr = True
                continue
            add_elem(row, ctx, MYDICT_C, MYDICT_S)


Parameters as key of ctx:
    default_XXX: value for field XXX if not in csv file
    trx_XXX: borderline translation dictionary for field XXX (1)
    company_name: company name to import data
    flavour: suffix to compose csv filename

Field in csv files:

(1) Borderline Translation dictionary:
Some fields require to convert source value into local Odoo value.
To do automatic translation supply a dictionary in form:
trx_XXX = {'original_value': 'Odoo_value', ...}
When field XXX in file csc contains value 'original_value' is written into
local Odoo DB as 'Odoo_value'.
"""
import sys

# import os
import time
import clodoo

try:
    from z0lib import z0lib
except ImportError:
    import z0lib
# import pdb


__version__ = "2.0.9"


msg_time = time.time()


def msg_burst(text):
    global msg_time
    t = time.time() - msg_time
    if t > 3:
        print(text)
        msg_time = time.time()


def get_company_id(ctx):
    model = 'res.company'
    company_name = ctx.get('company_name', 'La % Azienda')
    ids = clodoo.searchL8(ctx, model, [('name', 'ilike', company_name)])
    if not ids:
        ids = clodoo.searchL8(ctx, model, [('id', '>', 1)])
    if ids:
        return ids[0]
    else:
        return 1


def write_log(msg):
    fd = open('./import_products.log', 'a')
    fd.write('%s\n' % msg)
    fd.close()
    print(msg)


def add_elem(row, ctx, MYDICT):
    def cvt_val(ctx, field_name, value):
        x = 'trx_%s' % field_name
        if x in ctx:
            trx_dict = ctx[x]
            if value in trx_dict:
                value = trx_dict[value]
            else:
                value = value.strip()
        elif field_name == 'company_id':
            value = get_company_id(ctx)
        else:
            value = value.strip()
        return value

    def add_val(row, ctx, MYDICT, field_name):
        def_field_name = 'default_%s' % field_name
        if field_name in MYDICT:
            idx = MYDICT[field_name]
            if idx >= len(row):
                print('Invalid translation field %d' % idx)
                return
            vals[field_name] = cvt_val(ctx, field_name, row[idx])
        elif def_field_name in ctx:
            vals[field_name] = ctx[def_field_name]

    msg_burst('Reading %s ...' % row[MYDICT['name']])
    model = 'product.product'
    vals = {}
    add_val(row, ctx, MYDICT, 'name')
    for field_id in clodoo.searchL8(ctx, 'ir.model.fields', [('model', '=', model)]):
        field_name = clodoo.browseL8(ctx, 'ir.model.fields', field_id).name
        add_val(row, ctx, MYDICT, field_name)

    if 'name' not in vals:
        vals['name'] = 'PRODUCT'
    if 'company_id' not in vals:
        vals['company_id'] = get_company_id(ctx)

    ids = []
    if vals.get('code'):
        ids = clodoo.searchL8(ctx, model, [('default_code', '=', vals['code'])])
    if not ids and vals.get('default_code'):
        ids = clodoo.searchL8(ctx, model, [('default_code', '=', vals['default_code'])])
    if not ids:
        ids = clodoo.searchL8(ctx, model, [('name', '=', vals['name'])])
    if ids:
        product_id = ids[0]
        try:
            clodoo.writeL8(ctx, model, [product_id], vals)
        except BaseException:
            write_log("Cannot import %s" % vals['name'])
            try:
                clodoo.writeL8(ctx, model, [product_id], vals)
            except BaseException:
                pass
    else:
        try:
            product_id = clodoo.createL8(ctx, model, vals)
        except BaseException:
            write_log("Cannot create %s" % vals['name'])
            try:
                product_id = clodoo.createL8(ctx, model, vals)
            except BaseException:
                pass


def init_n_connect(flavour=None):
    title = 'Import products %s' % flavour or ''
    parser = z0lib.parseoptargs(
        title, "© 2017-2018 by SHS-AV s.r.l.", version=__version__
    )
    parser.add_argument('-h')
    parser.add_argument(
        "-c",
        "--config",
        help="configuration command file",
        dest="conf_fn",
        metavar="file",
        default='./import_products.config',
    )
    parser.add_argument(
        "-d",
        "--dbname",
        help="DB name",
        dest="db_name",
        metavar="file",
        default='demo8',
    )
    parser.add_argument(
        "-f",
        "--filename",
        help="Filename to import",
        dest="csv_fn",
        metavar="file",
        default=False,
    )
    parser.add_argument('-n')
    parser.add_argument('-q')
    parser.add_argument('-V')
    parser.add_argument('-v')
    # Connect to DB
    ctx = parser.parseoptargs(sys.argv[1:], apply_conf=False)
    ctx['flavour'] = flavour
    if not ctx['csv_fn']:
        if flavour:
            sfx = '_'
        else:
            sfx = ''
        ctx['csv_fn'] = 'products%s%s.csv' % (sfx, flavour or '')
    oerp, uid, ctx = clodoo.oerp_set_env(
        confn=ctx['conf_fn'], db=ctx['db_name'], ctx=ctx
    )
    return oerp, uid, ctx





# -*- coding: utf-8 -*-
#
# Copyright SHS-AV s.r.l. <http://www.zeroincombenze.org>)
#
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl).
#
#    All Rights Reserved
#
"""
    Clodoo Regression Test Suite
"""
import ast
import os
import sys
import time
from configparser import ConfigParser
from datetime import datetime
from multiprocessing import Process
from subprocess import PIPE, Popen

from zerobug import z0test

try:
    from clodoo import clodoo
except ImportError:
    import clodoo


__version__ = "2.0.9"


MODULE_ID = "clodoo"
TEST_FAILED = 1
TEST_SUCCESS = 0

MANIFEST_FILES = ["__manifest__.py", "__odoo__.py", "__openerp__.py", "__terp__.py"]


def get_server_path(odoo_full, odoo_version, travis_home):
    odoo_version = odoo_version.replace("/", "-")
    odoo_org, odoo_repo = odoo_full.split("/")
    server_dirname = "%s-%s" % (odoo_repo, odoo_version)
    server_path = os.path.join(travis_home, server_dirname)
    return server_path


def get_server_script(server_path):
    if os.path.isfile(os.path.join(server_path, "odoo-bin")):
        return "odoo-bin"
    elif os.path.isfile(os.path.join(server_path, "openerp-server")):
        return "openerp-server"
    return "Script not found!"


def get_script_path(server_path, script_name):
    script_path = os.path.join(server_path, "server")
    if os.path.isdir(script_path):
        script_path = os.path.join(server_path, "server", script_name)
    else:
        script_path = os.path.join(server_path, script_name)
    return script_path


def set_conf_data(addons_path, data_dir=None, logfile=None):
    data_dir = data_dir or os.path.expanduser(
        os.environ.get("DATA_DIR", os.path.expanduser("~/data_dir"))
    )
    conf_data = {
        "addons_path": addons_path,
        "data_dir": data_dir,
        "db_password": clodoo.decrypt('Wg".:gfL'),
    }
    if logfile:
        conf_data["logfile"] = logfile
    if os.uname()[1][0:3] == "shs":
        pid = os.getpid()
        if pid > 18000:
            rpcport = str(pid)
        else:
            rpcport = str(18000 + pid)
        conf_data["xmlrpc_port"] = rpcport
        conf_data["db_user"] = "odoo"
    return conf_data


def get_modules(path, depth=1):
    """Return modules of path repo (used in test_server.py)"""
    return sorted(list(get_modules_info(path, depth).keys()))


def get_modules_info(path, depth=1):
    """Return a digest of each installable module's manifest in path repo"""
    # Avoid empty basename when path ends with slash
    path = os.path.expanduser(path)
    if not os.path.basename(path):
        path = os.path.dirname(path)

    modules = {}
    if os.path.isdir(path) and depth > 0:
        for module in os.listdir(path):
            manifest_path = is_module(os.path.join(path, module))
            if manifest_path:
                try:
                    manifest = ast.literal_eval(open(manifest_path).read())
                except ImportError:
                    raise Exception("Wrong file %s" % manifest_path)
                if manifest.get("installable", True):
                    modules[module] = {
                        "application": manifest.get("application"),
                        "depends": manifest.get("depends") or [],
                        "auto_install": manifest.get("auto_install"),
                    }
            else:
                deeper_modules = get_modules_info(os.path.join(path, module), depth - 1)
                modules.update(deeper_modules)
    return modules


def is_module(path):
    """return False if the path doesn't contain an odoo module, and the full
    path to the module manifest otherwise"""

    path = os.path.expanduser(path)
    if not os.path.isdir(path):
        return False
    files = os.listdir(path)
    filtered = [x for x in files if x in (MANIFEST_FILES + ["__init__.py"])]
    if len(filtered) == 2 and "__init__.py" in filtered:
        return os.path.join(path, next(x for x in filtered if x != "__init__.py"))
    else:
        return False


def is_addons(path):
    res = get_modules(path) != []
    return res


def get_addons(path, depth=1):
    path = os.path.expanduser(path)
    if not os.path.exists(path) or depth < 0:
        return []
    res = []
    if is_addons(path):
        res.append(path)
    else:
        new_paths = [
            os.path.join(path, x)
            for x in sorted(os.listdir(path))
            if os.path.isdir(os.path.join(path, x))
        ]
        for new_path in new_paths:
            res.extend(get_addons(new_path, depth - 1))
    return res


def get_addons_path(
    travis_dependencies_dir, travis_base_dir, server_path, odoo_test_select
):
    addons_path_list = []
    for ldir in ("server/openerp", "openerp", "odoo"):
        if os.path.isdir(os.path.join(server_path, ldir, "addons")):
            addons_path_list = [os.path.join(server_path, ldir, "addons")]
            break
    addons_path_list.append(os.path.join(server_path, "addons"))
    if travis_base_dir:
        if odoo_test_select != "NO-CORE":
            addons_path_list.extend(get_addons(travis_base_dir))
    if travis_dependencies_dir:
        addons_path_list.extend(get_addons(travis_dependencies_dir))
    addons_path = ",".join(addons_path_list)
    return addons_path


def write_server_conf(data, version):
    fname_conf = os.path.expanduser("~/.openerp_serverrc")
    if not os.path.exists(fname_conf):
        fconf = open(fname_conf, "w")
        fconf.close()
    config = ConfigParser()
    config.read(fname_conf)
    if not config.has_section("options"):
        config["options"] = {}
    config["options"].update(data)
    with open(fname_conf, "w") as configfile:
        config.write(configfile)


def get_odoo_cmd(script_path, db=None, modules=None, loglevel=None):
    loglevel = loglevel or "info"
    cmd_odoo = [script_path, "--log-level=%s" % loglevel]
    if db:
        cmd_odoo += ["-d", db]
    if modules:
        cmd_odoo += ["--stop-after-init", "--init", ",".join(modules)]
    return cmd_odoo


def run_odoo_cmd(cmd_odoo):
    prc = Popen(cmd_odoo, stderr=PIPE, stdout=PIPE)
    res, err = prc.communicate()
    print(res)


def str_secret(obj):
    def str_list_secret(obj):
        obj_secret = []
        skip_next = False
        for param in obj:
            if skip_next:
                skip_next = False
                continue
            if param.startswith("--pass"):
                obj_secret.append(param.split("=")[0] + "=***")
                continue
            if param.startswith("--pwd"):
                obj_secret.append("--log-db=***")
                continue
            obj_secret.append(param)
        return obj_secret

    def str_dict_secret(obj):
        obj_secret = obj.copy()
        for param in obj:
            if param in (
                "db_password",
                "login_password",
                "crypt_password",
                "login2_password",
                "crypt2_password",
                "oneadm_pwd",
                "botadm_pwd",
                "admin_passwd",
            ):
                obj_secret[param] = "***"
        return obj_secret

    if isinstance(obj, (list, tuple)):
        return str_list_secret(obj)
    elif isinstance(obj, dict):
        return str_dict_secret(obj)
    return obj


def version():
    return __version__


class RegressionTest:
    def __init__(self, zarlib):
        self.Z = zarlib
        self.ctx = {}
        self.uid = False
        self.db = os.environ.get("MQT_TEST_DB", "clodoo_test")
        self.odoo_full = os.environ.get("ODOO_REPO", "zeroincombenze/OCB")
        self.odoo_version = os.environ.get("VERSION", "10.0")

    def check_4_db(self, dbname):
        cmd = ["psql"] + ["-Upostgres"] + ["-tl"]
        prc = Popen(cmd, stdin=PIPE, stdout=PIPE, stderr=PIPE)
        res, err = prc.communicate()
        dbname = " %s " % dbname
        if res.find(dbname) >= 0:
            return True
        else:
            return False

    def __test_01(self, z0ctx):
        sts = TEST_SUCCESS
        travis_debug_mode = eval(os.environ.get("TRAVIS_DEBUG_MODE", "0"))
        if not z0ctx["dry_run"]:
            travis_home = os.environ.get("HOME", os.path.expanduser("~"))
            print("Odoo repo=%s" % self.odoo_full)
            print("Odoo ver=%s" % self.odoo_version)
            server_path = get_server_path(
                self.odoo_full, self.odoo_version, travis_home
            )
            script_name = get_server_script(server_path)
            script_path = get_script_path(server_path, script_name)
            addons_path = get_addons_path(False, False, server_path, "ALL")
            self.logfile = os.path.join(
                os.environ.get("HOME", os.path.expanduser("~")),
                "odoo_%s.log" % self.odoo_version.replace(".", "-"),
            )
            conf_data = set_conf_data(addons_path, logfile=self.logfile)
            self.rpcport = conf_data.get("xmlrpc_port", 8069)
            write_server_conf(conf_data, self.odoo_version)
            cmd_odoo = get_odoo_cmd(script_path)
            print("script_name=%s" % script_name)
            print("script_path=%s" % script_path)
            print("addons_path=%s" % addons_path)
            print("conf_data=%s" % str_secret(conf_data))
            print("cmd_odoo=%s" % cmd_odoo)
            # run_odoo_cmd(cmd_odoo)
            prc = Process(target=run_odoo_cmd, args=(cmd_odoo,))
            prc.start()
            if os.environ.get("TRAVIS", "") == "true":
                time.sleep(6)
            else:
                time.sleep(3)
            if travis_debug_mode >= 2:
                pass
            self.uid, self.ctx = clodoo.oerp_set_env(
                db=self.db,
                ctx={
                    "oe_version": "*",
                    "no_login": True,
                    "xmlrpc_port": self.rpcport,
                    "conf_fn": "./no_filename.conf",
                },
            )
        else:
            self.ctx = {"oe_version": self.odoo_version}
        sts = self.Z.test_result(
            z0ctx, "connect(%s)" % self.db, self.odoo_version, self.ctx["oe_version"]
        )

        if not z0ctx["dry_run"]:
            self.ctx["test_unit_mode"] = True
            clodoo.act_drop_db(self.ctx)
            sts = clodoo.act_new_db(self.ctx)
        sts = self.Z.test_result(z0ctx, "new_db(%s)" % self.db, TEST_SUCCESS, sts)
        if not z0ctx["dry_run"]:
            res = self.check_4_db(self.db)
        else:
            res = True
        sts = self.Z.test_result(z0ctx, "_db(%s)" % self.db, True, res)
        if not z0ctx["dry_run"]:
            self.uid, self.ctx = clodoo.oerp_set_env(
                db=self.db,
                ctx={
                    "oe_version": self.ctx["oe_version"],
                    "xmlrpc_port": self.rpcport,
                    "conf_fn": "./no_filename.conf",
                },
            )
            self.ctx["test_unit_mode"] = True
            res = self.uid
        else:
            res = 1
        sts = self.Z.test_result(z0ctx, "login_%s" % self.db, 1, res)
        if not z0ctx["dry_run"]:
            prc.terminate()
        return sts

    def __test_02(self, z0ctx):
        sts = TEST_SUCCESS
        ctx = self.ctx
        if not z0ctx["dry_run"]:
            ids = clodoo.searchL8(ctx, "res.users", [])
        else:
            ids = [1]
        sts = self.Z.test_result(z0ctx, "searchL8(res.users)", True, 1 in ids)
        if sts == TEST_SUCCESS:
            model = "res.users"
            if not z0ctx["dry_run"]:
                user = clodoo.browseL8(ctx, "res.users", 1)
                user_id = user.id
            else:
                user_id = 1
            sts = self.Z.test_result(z0ctx, "browseL8(res.users)", 1, user_id)
        if sts == TEST_SUCCESS:
            if not z0ctx["dry_run"]:
                RES = clodoo.get_val_from_field(ctx, model, user, "login")
            else:
                RES = "admin"
            sts = self.Z.test_result(z0ctx, "user[login] (char)", "admin", RES)
        if sts == TEST_SUCCESS:
            if not z0ctx["dry_run"]:
                RES = clodoo.get_val_from_field(ctx, model, user, "active")
            else:
                RES = True
            sts = self.Z.test_result(z0ctx, "user[active] (bool)", True, RES)
        if sts == TEST_SUCCESS:
            if not z0ctx["dry_run"]:
                RES = clodoo.get_val_from_field(ctx, model, user, "company_id")
            else:
                RES = True
            sts = self.Z.test_result(z0ctx, "user[company_id] (m2o)", True, RES)
        if sts == TEST_SUCCESS:
            if not z0ctx["dry_run"]:
                RES = clodoo.get_val_from_field(ctx, model, user, "company_ids")
            else:
                RES = [1]
            sts = self.Z.test_result(z0ctx, "user[company_ids] (m2m)", [1], RES)
        if sts == TEST_SUCCESS:
            if not z0ctx["dry_run"]:
                RES = clodoo.get_val_from_field(
                    ctx, model, user, "company_ids", format="cmd"
                )
            else:
                RES = [(6, 0, [1])]
            sts = self.Z.test_result(
                z0ctx, "user[company_ids] (cmd)", [(6, 0, [1])], RES
            )
        if sts == TEST_SUCCESS:
            if not z0ctx["dry_run"]:
                RES = clodoo.get_val_from_field(ctx, model, user, "login_date")
            else:
                RES = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
            sts = self.Z.test_result(
                z0ctx,
                "user[login_date] (date[time])",
                datetime.now().strftime("%Y-%m-%d"),
                str(RES)[0:10],
            )
        if sts == TEST_SUCCESS:
            if not z0ctx["dry_run"]:
                RES = clodoo.get_val_from_field(
                    ctx, model, user, "login_date", format="cmd"
                )
            else:
                RES = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
            sts = self.Z.test_result(
                z0ctx,
                "user[login_date] (date[time])",
                datetime.now().strftime("%Y-%m-%d"),
                RES[0:10],
            )
        return sts

    def __test_03(self, z0ctx):
        sts = TEST_SUCCESS
        ctx = self.ctx
        if not z0ctx["dry_run"]:
            model = "ir.model"
            rec = clodoo.browseL8(
                ctx,
                model,
                clodoo.searchL8(ctx, model, [("model", "=", "res.users")])[0],
            )
            RES = clodoo.get_val_from_field(ctx, model, rec, "field_id")
        else:
            RES = [1, 2, 3]
        sts = self.Z.test_result(
            z0ctx,
            "ir_model[field_id] (o2m)",
            True,
            isinstance(RES, list) and len(RES) > 1,
        )
        if sts == TEST_SUCCESS:
            record_list = RES
            if not z0ctx["dry_run"]:
                model = "ir.model"
                RES = clodoo.get_val_from_field(
                    ctx, model, rec, "field_id", format="cmd"
                )
            else:
                RES = [(6, 0, record_list)]
            sts = self.Z.test_result(
                z0ctx, "ir_model[field_id] (cmd)", [(6, 0, record_list)], RES
            )
        return sts

    def __test_04(self, z0ctx):
        sts = TEST_SUCCESS
        ctx = self.ctx
        model = "res.users"
        vals = {"name": "admin", "partner_id": 1, "email": "myname@example.com"}
        for tgt_ver in ("12.0", "11.0", "10.0", "9.0", "8.0", "7.0", "6.1"):
            for name in vals:
                if not z0ctx["dry_run"]:
                    new_name, RES = clodoo.cvt_value_from_ver_to_ver(
                        ctx, model, name, vals[name], "7.0", tgt_ver
                    )
                else:
                    RES = new_name = True
                if name == "name":
                    TRES = vals[name]
                    sts = self.Z.test_result(z0ctx, "cvt(%s)" % name, TRES, RES)
                elif name == "partner_id":
                    if tgt_ver == "6.1":
                        TRES = False
                    else:
                        TRES = vals[name]
                    sts = self.Z.test_result(z0ctx, "cvt(%s)" % name, TRES, RES)
                elif name == "email":
                    if tgt_ver == "6.1":
                        TRES = "user_email"
                    else:
                        TRES = name
                    sts = self.Z.test_result(z0ctx, "cvt(%s)" % name, TRES, new_name)
        return sts

    def __test_05(self, z0ctx):
        sts = TEST_SUCCESS
        ctx = self.ctx
        model = "res.users"
        vals = {"name": "admin", "partner_id": 1, "email": "myname@example.com"}
        for tgt_ver in ("12.0", "11.0", "10.0", "9.0", "8.0", "7.0", "6.1"):
            if not z0ctx["dry_run"]:
                new_vals = clodoo.cvt_from_ver_2_ver(ctx, model, "7.0", tgt_ver, vals)
            else:
                new_vals = vals
            if tgt_ver == "6.1":
                TRES = ["name", "user_email"]
            else:
                TRES = sorted(vals.keys())
            sts = self.Z.test_result(
                z0ctx, "cvt_rec(res.users)", TRES, sorted(new_vals.keys())
            )
        return sts


#
# Run main if executed as a script
if __name__ == "__main__":
    exit(
        z0test.main_local(
            z0test.parseoptest(sys.argv[1:], version=version()), RegressionTest
        )
    )





# -*- coding: utf-8 -*-
#
# Copyright SHS-AV s.r.l. <http://www.zeroincombenze.it>)
#
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl).
#
#    All Rights Reserved
#
"""
    Clodoo Regression Test Suite
"""
import os
import sys

# import time
from configparser import ConfigParser

# from datetime import datetime

from zerobug import z0test

try:
    from clodoo import clodoo
except ImportError:
    import clodoo


__version__ = "2.0.9"

ODOO_VERSION_TO_TEST = ("16.0", "12.0", "10.0", "8.0", "7.0")


def version():
    return __version__


class RegressionTest:
    def __init__(self, zarlib):
        self.Z = zarlib

    def setup(self, z0ctx):
        print("Connection test: it works only if odoo instances are running!")
        self.test_data_dir = os.path.join(self.Z.testdir, 'res')
        if not os.path.isdir(self.test_data_dir):
            os.mkdir(self.test_data_dir)
        config = ConfigParser()
        default_confn = os.path.abspath(
            os.path.join(os.path.dirname(__file__), 'default-odoo.conf')
        )
        config.read(default_confn)
        self.version_ctx = {}
        self.version_default = {}
        for odoo_version in ODOO_VERSION_TO_TEST:
            odoo_major = int(odoo_version.split(".")[0])
            values = {
                param: config["options"].get("%s%d" % (param, odoo_major))
                or (val % odoo_major if "%" in val else val)
                for (param, val) in (
                    ("oe_version", odoo_version),
                    ("db_host", "localhost"),
                    ("db_port", "False"),
                    ("db_template", "template1"),
                    ("db_user", "odoo%d"),
                    ("login_user", "admin"),
                    ("password", "admin"),
                    ("db_name", "test%d"),
                    (
                        "xmlrpc_port" if odoo_major < 10 else "http_port",
                        str(
                            clodoo.build_odoo_param("RPCPORT", odoo_version, multi=True)
                        ),
                    ),
                    ("psycopg2", "1"),
                    ("svc_protocol", "xmlrpc" if odoo_major < 10 else "jsonrpc"),
                    ("admin_password", "admin"),
                    ("tmp_database", "test_clodoo_%d"),
                    ("admin_password", "admin"),
                )
            }
            if odoo_major >= 10:
                values["longpolling_port"] = str(
                    clodoo.build_odoo_param("LPPORT", odoo_version, multi=True)
                )
            self.version_default[odoo_version] = values
            print(
                "  ... Odoo %s should be run on %s:%s, DB=%s for %s login user!"
                % (
                    odoo_version,
                    values["db_host"],
                    values.get("http_port") or values["xmlrpc_port"],
                    values["db_name"],
                    values["login_user"],
                )
            )

    def test_01(self, z0ctx):
        sts = 0
        for odoo_version in ODOO_VERSION_TO_TEST:
            odoo_major = int(odoo_version.split(".")[0])
            database = self.version_default[odoo_version]["db_name"]
            confn = os.path.join(self.test_data_dir, 'odoo%s.conf' % odoo_major)
            if not z0ctx.get('dry_run', False):
                config = ConfigParser()
                config["options"] = {}
                config["options"].update(self.version_default[odoo_version])
                config.write(open(confn, "w"))
            uid, ctx = clodoo.oerp_set_env(ctx={}, confn=confn, db=database)
            self.version_ctx[odoo_version] = ctx
            sts += self.Z.test_result(
                z0ctx,
                "Connect %s/%s (pypi: %s)" % (database, odoo_version, ctx["pypi"]),
                uid > 0,
                True,
            )
        return sts

    def test_02(self, z0ctx):
        sts = 0
        resource = "res.partner"
        partner_name = "Test clodoo"
        partner_updated = "Test updated partner"
        lang = "en_GB"

        for odoo_version in ODOO_VERSION_TO_TEST:
            odoo_major = int(odoo_version.split(".")[0])
            database = self.version_default[odoo_version]["db_name"]
            ids = []
            if not z0ctx.get('dry_run', False):
                ids = clodoo.searchL8(self.version_ctx[odoo_version], resource, [])
            sts += self.Z.test_result(
                z0ctx, "(%s) SearchL8 %s" % (odoo_version, database), len(ids) > 0, True
            )

            id = False
            if not z0ctx.get('dry_run', False):
                id = clodoo.createL8(
                    self.version_ctx[odoo_version], resource, {"name": partner_name}
                )
            sts += self.Z.test_result(
                z0ctx, "    CreateL8 %s" % database, len(ids) > 0, True
            )
            ids = clodoo.searchL8(
                self.version_ctx[odoo_version], resource, [("id", "=", id)]
            )
            sts += self.Z.test_result(
                z0ctx, "    SearchL8 %s" % database, len(ids) > 0, True
            )

            partner = None
            if not z0ctx.get('dry_run', False):
                partner = clodoo.browseL8(self.version_ctx[odoo_version], resource, id)
            sts += self.Z.test_result(
                z0ctx,
                "    BrowseL8 %s" % database,
                partner.name if partner else partner,
                partner_name,
            )

            if not z0ctx.get('dry_run', False):
                clodoo.writeL8(
                    self.version_ctx[odoo_version],
                    resource,
                    id,
                    {"name": partner_updated},
                )
            sts += self.Z.test_result(
                z0ctx,
                "    WriteL8 %s" % database,
                clodoo.browseL8(self.version_ctx[odoo_version], resource, id).name,
                partner_updated,
            )

            if not z0ctx.get('dry_run', False):
                clodoo.unlinkL8(self.version_ctx[odoo_version], resource, id)
                ids = clodoo.searchL8(
                    self.version_ctx[odoo_version], resource, [("id", "=", id)]
                )
            sts += self.Z.test_result(z0ctx, "    UnlinkL8 %s" % database, ids, [])

            if not z0ctx.get('dry_run', False):
                ids = clodoo.searchL8(
                    self.version_ctx[odoo_version], "res.lang", [("code", "=", lang)]
                )
                if not ids:
                    ids = clodoo.searchL8(
                        self.version_ctx[odoo_version],
                        "res.lang",
                        [("code", "=", lang), ("active", "=", False)],
                    )
                if odoo_major >= 16:
                    id = clodoo.createL8(
                        self.version_ctx[odoo_version],
                        "base.language.install",
                        {
                            "lang_ids": ids,
                            "overwrite": False,
                        },
                    )
                else:
                    id = clodoo.createL8(
                        self.version_ctx[odoo_version],
                        "base.language.install",
                        {
                            "lang": lang,
                            "overwrite": False,
                        },
                    )
                act = clodoo.executeL8(
                    self.version_ctx[odoo_version],
                    "base.language.install",
                    "lang_install",
                    [id],
                )
                sts += self.Z.test_result(
                    z0ctx, "    lang_install(%s)" % ids, isinstance(act, dict), True
                )

                if odoo_major < 10:
                    id = clodoo.createL8(
                        self.version_ctx[odoo_version],
                        "base.update.translations",
                        {
                            "lang": lang,
                        },
                    )
                    act = clodoo.executeL8(
                        self.version_ctx[odoo_version],
                        "base.update.translations",
                        "act_update",
                        [id],
                    )
                    sts += self.Z.test_result(
                        z0ctx,
                        "    lang.act_update(%s)" % ids,
                        isinstance(act, dict),
                        True,
                    )
        return sts


#
# Run main if executed as a script
if __name__ == "__main__":
    exit(
        z0test.main_local(
            z0test.parseoptest(sys.argv[1:], version=version()), RegressionTest
        )
    )





# -*- coding: utf-8 -*-
#
# Copyright SHS-AV s.r.l. <http://www.zeroincombenze.org>)
#
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl).
#
#    All Rights Reserved
#
"""
    Clodoo Regression Test Suite
"""
from __future__ import print_function, unicode_literals

import sys
from datetime import date

from past.builtins import basestring

try:
    from clodoo.clodoo import (
        check_4_actions,
        create_act_list,
        do_single_action,
        isaction,
    )
    from clodoo.clodoocore import (
        _get_model_bone,
        _get_model_code,
        _get_model_name,
        _import_file_dbtype,
        _import_file_model,
        eval_value,
        get_query_id,
        import_file_get_hdr,
        model_has_company,
    )
except BaseException:
    from clodoo import check_4_actions, create_act_list, do_single_action, isaction
    from clodoocore import (
        _get_model_bone,
        _get_model_code,
        _get_model_name,
        _import_file_dbtype,
        _import_file_model,
        model_has_company,
        eval_value,
        get_query_id,
        import_file_get_hdr,
    )

from zerobug import z0test

__version__ = "2.0.9"

MODULE_ID = "clodoo"
TEST_FAILED = 1
TEST_SUCCESS = 0
MY_ACT = "my_actions"

CID = 9
DEF_CID = 1
REF_ID = 1
ID_0 = 11
ID_1 = 1
ID_I = 4
ID_C_0 = 13
MARK = "Zeroincombenze®"
CTRY_IT = 110
PARTNER_DEF = 2
USER_DEF = 3


def version():
    return __version__


class Oerp:
    """oerplib simulator for test execution
    It simulates:
    - res.zero: model w/o company (key is name, 1 record)
    - res.one: model with company (key is name, 1 record)
    - res.two: model w/o company (key is name, many records)
    - res.three: model with company (key is name, may records)
    - res.four: model w/o company (key is other)
    - res.five: model with company (key is other)
    """

    def __init__(self):
        self.id = 0
        self.model = ""
        self.value = ""
        # self.db = type('', (), {})
        self.db = {}

        # system table: ir.model
        model = "ir.model"
        self.db["model"] = {}
        self.db[model] = {
            1: {
                "model": "res.zero",
                "module": "test",
                "name": "res.zero",
                "field_ids": [],
            },
            2: {
                "model": "res.one",
                "module": "test",
                "name": "res.one",
                "field_ids": [],
            },
            3: {
                "model": "res.three",
                "module": "test",
                "name": "res.three",
                "field_ids": [],
            },
            4: {
                "model": "res.four",
                "module": "test",
                "name": "res.four",
                "field_ids": [],
            },
            5: {
                "model": "res.five",
                "module": "test",
                "name": "res.five",
                "field_ids": [],
            },
        }

        # system table: ir.model.fields
        model = "ir.model.fields"
        self.db["model"] = {}
        self.db[model] = {
            1: {
                "model": "res.zero",
                "modules": "test",
                "name": "name",
                "readonly": False,
                "relation": False,
                "required": True,
                "ttype": "char",
            },
            2: {
                "model": "res.zero",
                "modules": "test",
                "name": "ref",
                "readonly": False,
                "relation": False,
                "required": True,
                "ttype": "integer",
            },
            3: {
                "model": "res.zero",
                "modules": "test",
                "name": "description",
                "readonly": False,
                "relation": False,
                "required": True,
                "ttype": "text",
            },
            4: {
                "model": "res.one",
                "modules": "test",
                "name": "name",
                "readonly": False,
                "relation": False,
                "required": True,
                "ttype": "char",
            },
            5: {
                "model": "res.one",
                "modules": "test",
                "name": "ref0",
                "readonly": False,
                "relation": False,
                "required": True,
                "ttype": "integer",
            },
            6: {
                "model": "res.one",
                "modules": "test",
                "name": "company_id",
                "readonly": False,
                "relation": "res.company",
                "required": True,
                "ttype": "many2one",
            },
            7: {
                "model": "res.two",
                "modules": "test",
                "name": "name",
                "readonly": False,
                "relation": False,
                "required": True,
                "ttype": "char",
            },
            8: {
                "model": "res.two",
                "modules": "test",
                "name": "country_id",
                "readonly": False,
                "relation": "res.country",
                "required": True,
                "ttype": "many2one",
            },
            9: {
                "model": "res.three",
                "modules": "test",
                "name": "name",
                "readonly": False,
                "relation": False,
                "required": True,
                "ttype": "char",
            },
            10: {
                "model": "res.three",
                "modules": "test",
                "name": "ref0",
                "readonly": False,
                "relation": False,
                "required": True,
                "ttype": "integer",
            },
            11: {
                "model": "res.three",
                "modules": "test",
                "name": "company_id",
                "readonly": False,
                "relation": "res.company",
                "required": True,
                "ttype": "many2one",
            },
        }

        # system table: ir.model.data
        model = "ir.model.data"
        self.db["model"] = {}
        self.db[model] = {
            1: {
                "model": "res.company",
                "module": "base",
                "name": "main_company",
                "res_id": ID_0,
            }
        }

        # date test table: res.zero
        # name, ref, description
        model = "res.zero"
        self.db["model"] = {}
        self.db[model] = {ID_0: {"name": "myname", "ref": REF_ID, "description": MARK}}

        # date test table: res.one
        # name, ref0, company_id
        model = "res.one"
        self.db["model"] = {}
        self.db[model] = {ID_1: {"name": "myname", "ref0": ID_0, "company_id": CID}}

        # date test table: res.two
        # name, country_id
        model = "res.two"
        self.db["model"] = {}
        self.db[model] = {REF_ID: {"name": "myname", "country_id": CTRY_IT}}

        # date test table: res.three
        # name, ref0, company_id
        model = "res.three"
        self.db["model"] = {}
        self.db[model] = {ID_1: {"name": "name àèìòù", "ref0": ID_0, "company_id": CID}}

    def comp_tuple(self, left, op, right):
        """Apply for op between 2 operands
        op may be '==', '!=' or 'ilike'
        """
        if op == "=":
            return left == right
        elif op == "!=":
            return left != right
        elif op == "ilike":
            return left.lower().find(right.lower()) >= 0
        return False

    def eval_where(self, model, where):
        """Evaluate sql where condition"""
        if model == "":
            raise IOError("*** No model supplied!!!")
        if model not in self.db:
            raise IOError("*** Invalid model %s !!!" % model)
        if len(where) == 1:
            return where[0]
        left = where[0]
        op = where[1]
        right = where[2]
        res = []
        if left == "id":
            if right in self.db[model]:
                res = [right]
            else:
                res = []
        else:
            for id in self.db[model]:
                if left not in self.db[model][id]:
                    raise IOError("*** Invalid field %s in model %s!!!" % (left, model))
                if self.comp_tuple(self.db[model][id][left], op, right):
                    res.append(id)
        return res

    def search(self, model, where, order=None, context=None):
        """Simulate search function"""
        if model == "":
            raise IOError("*** No model supplied!!!")
        if model not in self.db:
            raise IOError("*** Invalid " + model + " model!!!")
        res = [id for id in self.db[model]]
        while where:
            cond = self.eval_where(model, where.pop(0))
            if cond == "|":
                res = list(
                    set(res)
                    & (
                        set(self.eval_where(model, where.pop(0)))
                        | set(self.eval_where(model, where.pop(0)))
                    )
                )
            elif cond == "&":
                res = list(
                    set(res)
                    & set(self.eval_where(where.pop(0)))
                    & set(self.eval_where(where.pop(0)))
                )
            elif isinstance(cond, (list, tuple)):
                res = list(set(res) & set(cond))
            elif isinstance(cond, (int, float)):
                if id not in res:
                    res.append(cond)
        return res

    def browse1(self, model, id, context=None):
        """Simulate browse function on single record"""
        if model == "":
            raise IOError("*** No model supplied!!!")
        if model not in self.db:
            raise IOError("*** Invalid %s model!!!" % model)
        if id not in self.db[model]:
            return None
        for field in self.db[model][id]:
            setattr(self, field, self.db[model][id][field])
        return self

    def browse(self, model, ids, context=None):
        """Simulate browse on 1 or more records"""
        if isinstance(ids, list):
            res = []
            for id in ids:
                res.append(self.browse1(model, id, context=context))
            return res
        else:
            return self.browse1(model, ids, context=context)


class Csv:
    """csv simulator for test execution"""

    def _init(self):
        self.fieldnames = ""


class Conf:
    """confparser simulator for test execution"""

    def _init(self):
        pass

    def has_section(self, section):
        if section == MY_ACT:
            return True
        else:
            return False

    def has_option(self, section, name):
        if section == MY_ACT and name == "actions":
            return True
        else:
            return False

    def get(self, section, name):
        if section == MY_ACT and name == "actions":
            return "unit_test"
        else:
            return False


class RegressionTest:
    def __init__(self, zarlib):
        self.Z = zarlib
        self.oerp = False
        self.dbname = False

    def new_db(self, dbname):
        if dbname != self.dbname:
            self.oerp = Oerp()
            self.dbname = dbname
        elif not self.oerp:
            self.oerp = Oerp()
        return self.oerp

    def init_test(self, model):
        oerp = self.new_db("clodoo_test")
        o_model = {}
        ctx = {}
        ctx["odoo_session"] = oerp
        ctx["server_version"] = "8.0"
        ctx["majver"] = int(ctx["server_version"].split(".")[0])
        ctx["svc_protocol"] = "xmlrpc"
        if model not in ("res.zero", "res.two", "res.four"):
            ctx["def_company_id"] = DEF_CID
            ctx["company_id"] = CID
        else:
            o_model["hide_cid"] = True
        ctx["def_partner_id"] = PARTNER_DEF
        ctx["def_user_id"] = USER_DEF
        ctx["def_country_id"] = CTRY_IT
        ctx["panda"] = "P"
        ctx["P" + str(ctx["def_partner_id"])] = "PP"
        ctx["panda" + ctx["P" + str(ctx["def_partner_id"])]] = "PPP"
        conf_obj = Conf()
        ctx["_conf_obj"] = conf_obj
        ctx["_opt_obj"] = None
        ctx["actions_db"] = ""
        ctx["actions_mc"] = ""
        ctx["test_unit_mode"] = True
        if model in ("res.zero", "res.one"):
            o_model["db_type"] = "Z"
        elif model in ("res.two",):
            o_model["db_type"] = "T"
        else:
            o_model["db_type"] = "C"
        o_model["name"] = "name"
        o_model["code"] = "name"
        o_model["model"] = model
        csv_fn = ""
        row = {}
        row["undef_name"] = ""
        csv_obj = Csv()
        o_model = import_file_get_hdr(ctx, o_model, csv_obj, csv_fn, row)
        return oerp, ctx, o_model, csv_obj, csv_fn, row

    def test_01(self, z0ctx):
        msg = "_get_model_bone"
        sts = TEST_SUCCESS
        for model_name in ("res.zero", "res.one", "res.two", "res.four"):
            oerp, ctx, o_model, csv_obj, csv_fn, row = self.init_test(model_name)
            model, hide_cid = _get_model_bone(ctx, o_model)
            if sts == TEST_SUCCESS:
                sts = self.Z.test_result(z0ctx, msg, model_name, model)
            if sts == TEST_SUCCESS:
                if model in ("res.zero", "res.two", "res.four"):
                    sts = self.Z.test_result(z0ctx, msg, True, hide_cid)
                else:
                    sts = self.Z.test_result(z0ctx, msg, False, hide_cid)

        for model_name in ("res.zero", "res.one", "res.two"):
            oerp, ctx, o_model, csv_obj, csv_fn, row = self.init_test(model_name)
            del o_model["hide_cid"]
            model, hide_cid = _get_model_bone(ctx, o_model)
            sts = self.Z.test_result(z0ctx, msg, model_name, model)
            if sts == TEST_SUCCESS:
                if model in ("res.zero", "res.two", "res.four"):
                    sts = self.Z.test_result(z0ctx, msg, True, hide_cid)
                else:
                    sts = self.Z.test_result(z0ctx, msg, False, hide_cid)

        msg = "_import_file_model"
        for csv_fn in ("res_zero.csv", "res-zero.csv", "res.zero.csv"):
            o_model = {}
            if sts == TEST_SUCCESS:
                model, hide_cid = _import_file_model(ctx, o_model, csv_fn)
                sts = self.Z.test_result(z0ctx, msg, "res.zero", model)
            if sts == TEST_SUCCESS:
                sts = self.Z.test_result(z0ctx, msg, False, hide_cid)
            if sts == TEST_SUCCESS:
                o_model["model"] = "res.one"
                model, hide_cid = _import_file_model(ctx, o_model, csv_fn)
                sts = self.Z.test_result(z0ctx, msg, "res.one", model)
            if sts == TEST_SUCCESS:
                sts = self.Z.test_result(z0ctx, msg, False, "hide_cid" in o_model)
        return sts

    def test_02(self, z0ctx):
        o_model = {}
        msg = "_get_model_name"
        sts = TEST_SUCCESS
        for n in ("id", "name", "code", "other"):
            if sts == TEST_SUCCESS:
                fields = ["a", "b", n, "z"]
                name = _get_model_name(fields, o_model)
                if n == "other" or n == "id":
                    sts = self.Z.test_result(z0ctx, msg, "name", name)
                else:
                    sts = self.Z.test_result(z0ctx, msg, n, name)

        fields = ["id", "name", "code", "my_name", "other"]
        for n in ("id", "name", "code", "my_name", "other"):
            if sts == TEST_SUCCESS:
                o_model["model_name"] = n
                name = _get_model_name(fields, o_model)
                sts = self.Z.test_result(z0ctx, msg, n, name)

        o_model = {}
        for n in ("id", "name", "code", "my_name", "other"):
            if sts == TEST_SUCCESS:
                o_model["name"] = n
                name = _get_model_name(fields, o_model)
                sts = self.Z.test_result(z0ctx, msg, n, name)
        return sts

    def test_03(self, z0ctx):
        o_model = {}
        msg = "_get_model_code"
        sts = TEST_SUCCESS
        for n in ("id", "name", "code", "other"):
            if sts == TEST_SUCCESS:
                fields = ["a", "b", n, "z"]
                code = _get_model_code(fields, o_model)
                if n == "other":
                    sts = self.Z.test_result(z0ctx, msg, "name", code)
                else:
                    sts = self.Z.test_result(z0ctx, msg, n, code)

        fields = ["id", "name", "code", "my_name", "other"]
        for n in ("id", "name", "code", "my_name", "other"):
            if sts == TEST_SUCCESS:
                o_model["model_code"] = n
                code = _get_model_code(fields, o_model)
                sts = self.Z.test_result(z0ctx, msg, n, code)

        o_model = {}
        for n in ("id", "name", "code", "my_name", "other"):
            if sts == TEST_SUCCESS:
                o_model["code"] = n
                code = _get_model_code(fields, o_model)
                sts = self.Z.test_result(z0ctx, msg, n, code)

        return sts

    def test_04(self, z0ctx):
        o_model = {}
        msg = "_import_file_dbtype"
        csv_fn = "res-zero.csv"
        sts = TEST_SUCCESS
        for n in ("db_type", "id", "name", "code", "other"):
            if sts == TEST_SUCCESS:
                fields = ["a", "b", n, "z"]
                name = _import_file_dbtype(o_model, fields, csv_fn)
                if n != "db_type":
                    sts = self.Z.test_result(z0ctx, msg, False, name)
                else:
                    sts = self.Z.test_result(z0ctx, msg, n, name)

        fields = ["db_type", "id", "name", "code", "my_name", "other"]
        for n in ("id", "name", "code", "my_name"):
            if sts == TEST_SUCCESS:
                o_model["db_type"] = n
                name = _import_file_dbtype(o_model, fields, csv_fn)
                sts = self.Z.test_result(z0ctx, msg, n, name)
        return sts

    def test_05(self, z0ctx):
        oerp, ctx, o_model, csv_obj, csv_fn, row = self.init_test(None)
        msg = "eval_value"
        sts = TEST_SUCCESS

        name = "code"
        value = "=${panda}"
        res = eval_value(ctx, o_model, name, value)
        tres = "P"
        sts = self.Z.test_result(z0ctx, msg, tres, res)
        if sts == TEST_SUCCESS:
            value = "${panda}"
            res = eval_value(ctx, o_model, name, value)
            tres = "P"
            sts = self.Z.test_result(z0ctx, msg, tres, res)
        if sts == TEST_SUCCESS:
            value = "=${panda}${def_partner_id}"
            res = eval_value(ctx, o_model, name, value)
            tres = "P" + str(ctx["def_partner_id"])
            sts = self.Z.test_result(z0ctx, msg, tres, res)
        if sts == TEST_SUCCESS:
            value = "${panda}${def_partner_id}"
            res = eval_value(ctx, o_model, name, value)
            tres = "P" + str(ctx["def_partner_id"])
            sts = self.Z.test_result(z0ctx, msg, tres, res)
        if sts == TEST_SUCCESS:
            value = "=${${panda}${def_partner_id}}"
            res = eval_value(ctx, o_model, name, value)
            tres = "PP"
            sts = self.Z.test_result(z0ctx, msg, tres, res)
        if sts == TEST_SUCCESS:
            value = "=${panda${${panda}${def_partner_id}}}"
            res = eval_value(ctx, o_model, name, value)
            tres = "PPP"

        for name in ("db_type", "id", "name", "code", "my_name", "other"):
            msg = "eval_value(" + name + ")"
            for value in (b"abc", "def", True, False, 13715, 4.5, date(1959, 6, 26)):
                if sts == TEST_SUCCESS:
                    res = eval_value(ctx, o_model, name, value)
                    if value == b"abc":
                        sts = self.Z.test_result(z0ctx, msg, "abc", res)
                    else:
                        sts = self.Z.test_result(z0ctx, msg, value, res)

            for v in ctx:
                if sts == TEST_SUCCESS:
                    value = "=${" + v + "}"
                    res = eval_value(ctx, o_model, name, value)
                    if isinstance(ctx[v], (basestring, bool, int, float)):
                        sts = self.Z.test_result(z0ctx, msg, ctx[v], res)

                if sts == TEST_SUCCESS:
                    value = "=0-${" + v + "}-9"
                    if isinstance(ctx[v], basestring):
                        tres = "0-" + ctx[v] + "-9"
                    elif isinstance(ctx[v], (bool, int, float)):
                        tres = "0-" + str(ctx[v]) + "-9"
                    else:
                        tres = "0--9"
                    res = eval_value(ctx, o_model, name, value)
                    sts = self.Z.test_result(z0ctx, msg, tres, res)

            for model_name in ("res.zero", "res.one", "res.two", "res.three"):
                oerp, ctx, o_model, csv_obj, csv_fn, row = self.init_test(model_name)
                if model_name == "res.three":
                    value = model_name + "::name àèìòù"
                elif model_name in ("res.zero", "res.two", "res.four"):
                    value = model_name + ":myname"
                else:
                    value = model_name + "::myname"
                if sts == TEST_SUCCESS:
                    res = eval_value(ctx, o_model, name, value)
                    sts = self.Z.test_result(z0ctx, msg, value, res)
                if sts == TEST_SUCCESS:
                    if name == "id":
                        value = "=${" + model_name
                    elif name == "name" or name == "db_type":
                        value = "=${" + model_name + "[" + name + "]"
                    else:
                        value = "=${" + model_name + "[id]"
                    if model_name == "res.three":
                        value = value + "::name àèìòù}"
                    elif model_name in ("res.zero", "res.two", "res.four"):
                        value = value + ":myname}"
                    else:
                        value = value + "::myname}"
                    res = eval_value(ctx, o_model, name, value)
                    if name == "name" and model_name == "res.three":
                        sts = self.Z.test_result(z0ctx, msg, "name àèìòù", res)
                    elif name == "name":
                        sts = self.Z.test_result(z0ctx, msg, "myname", res)
                    elif name == "db_type":
                        if model_name in ("res.zero", "res.one"):
                            tres = "Z"
                        elif model_name in ("res.two",):
                            tres = "T"
                        else:
                            tres = "C"
                        sts = self.Z.test_result(z0ctx, msg, tres, res)
                    else:
                        if value[0:11] == "=${res.zero":
                            sts = self.Z.test_result(z0ctx, msg, ID_0, res)
                        elif value[0:10] == "=${res.one":
                            sts = self.Z.test_result(z0ctx, msg, ID_1, res)
                        elif value[0:10] == "=${res.two":
                            sts = self.Z.test_result(z0ctx, msg, REF_ID, res)
                        elif value[0:12] == "=${res.three":
                            sts = self.Z.test_result(z0ctx, msg, ID_1, res)
                        else:
                            sts = self.Z.test_result(z0ctx, msg, False, True)
            if sts == TEST_SUCCESS:
                value = "base.main_company"
                name = "company_id"
                res = eval_value(ctx, o_model, name, value)
                sts = self.Z.test_result(z0ctx, msg, ID_0, res)
        return sts

    def test_06(self, z0ctx):
        oerp, ctx, o_model, csv_obj, csv_fn, row = self.init_test("res.one")
        msg = "eval_value"
        sts = TEST_SUCCESS
        for x in (("name", "my_name"), ("other", "=${res.zero:myname}")):
            if sts == TEST_SUCCESS:
                name = x[0]
                value = x[1]
                res = eval_value(ctx, o_model, name, value)
                if name == "name":
                    tres = "my_name"
                elif name == "other":
                    tres = ID_0
                sts = self.Z.test_result(z0ctx, msg, tres, res)
            if sts == TEST_SUCCESS:
                name = "ref0"
                value = "=${res.zero(name,ref):myname," + str(REF_ID) + "}"
                res = eval_value(ctx, o_model, name, value)
                sts = self.Z.test_result(z0ctx, msg, ID_0, res)
            if sts == TEST_SUCCESS:
                name = "ref0"
                value = "=${res.zero(name,ref)[description]:myname," + str(REF_ID) + "}"
                res = eval_value(ctx, o_model, name, value)
                sts = self.Z.test_result(z0ctx, msg, MARK, res)
            if sts == TEST_SUCCESS:
                name = "ref0"
                value = "=${company_id+1}"
                res = eval_value(ctx, o_model, name, value)
                sts = self.Z.test_result(z0ctx, msg, CID + 1, res)

            if sts == TEST_SUCCESS:
                name = "ref0"
                value = "=[(6,0,${company_id})]"
                res = eval_value(ctx, o_model, name, value)
                tres = [(6, 0, CID)]
                sts = self.Z.test_result(z0ctx, msg, tres, res)

            if sts == TEST_SUCCESS:
                name = "ref0"
                value = "=None"
                res = eval_value(ctx, o_model, name, value)
                if res is not None:
                    tres = True
                else:
                    tres = res
                sts = self.Z.test_result(z0ctx, msg, tres, res)

        if sts == TEST_SUCCESS:
            name = "name"
            value = "=${res.two(country_id,name):" + ",myname}"
            res = eval_value(ctx, o_model, name, value)
            sts = self.Z.test_result(z0ctx, msg, REF_ID, res)

        if sts == TEST_SUCCESS:
            name = "name"
            value = "=${res.two(country_id,name):" + ",not_exists}"
            res = eval_value(ctx, o_model, name, value)
            sts = self.Z.test_result(z0ctx, msg, "", res)

        if sts == TEST_SUCCESS:
            name = "name"
            value = (
                "=${res.one(company_id,ref0)::" + "${company_id},${res.zero:myname}}"
            )
            res = eval_value(ctx, o_model, name, value)
            sts = self.Z.test_result(z0ctx, msg, ID_1, res)
        return sts

    def test_07(self, z0ctx):
        msg = "model_has_company"
        sts = TEST_SUCCESS
        for model in ("res.zero", "res.one", "res.two"):
            if sts == TEST_SUCCESS:
                oerp, ctx, o_bones, csv_obj, csv_fn, row = self.init_test(model)
                res = model_has_company(ctx, model)
                if model == "res.zero":
                    tres = False
                elif model == "res.one":
                    tres = True
                elif model == "res.two":
                    tres = False
                sts = self.Z.test_result(z0ctx, msg, tres, res)

    def test_08(self, z0ctx):
        msg = "get_query_id"
        sts = TEST_SUCCESS
        for model in ("res.zero", "res.one", "res.two"):
            if sts == TEST_SUCCESS:
                oerp, ctx, o_bones, csv_obj, csv_fn, row = self.init_test(model)
                row["id"] = ""
                row["name"] = "myname"
                row["other"] = "=${res.one:myname}"
                res = get_query_id(ctx, o_bones, row)
                if model == "res.zero":
                    tres = [ID_0]
                elif model == "res.one":
                    tres = [ID_1]
                elif model == "res.two":
                    tres = [REF_ID]
                sts = self.Z.test_result(z0ctx, msg, tres, res)

            if model == "res.zero":
                if sts == TEST_SUCCESS:
                    o_bones["code"] = "ref"
                    row["ref"] = "=${res.two:~myname}"
                    res = get_query_id(ctx, o_bones, row)
                    sts = self.Z.test_result(z0ctx, msg, [ID_0], res)
                if sts == TEST_SUCCESS:
                    row["id"] = "base.main_company"
                    row["name"] = "my_name"
                    row["other"] = "hello world"
                    res = get_query_id(ctx, o_bones, row)
                    sts = self.Z.test_result(z0ctx, msg, [ID_0], res)

            if model in ("res.zero", "res.two"):
                if sts == TEST_SUCCESS:
                    oerp, ctx, o_bones, csv_obj, csv_fn, row = self.init_test(model)
                    o_bones["repl_by_id"] = True
                    row["id"] = ""
                    res = get_query_id(ctx, o_bones, row)
                    sts = self.Z.test_result(z0ctx, msg, [], res)

                for _i in range(2):
                    if sts == TEST_SUCCESS:
                        # repeat 2 time to check side effects
                        if model == "res.zero":
                            row["id"] = ID_0
                        elif model == "res.one":
                            row["id"] = ID_1
                        elif model == "res.two":
                            row["id"] = REF_ID
                        elif model == "res.three":
                            row["id"] = ID_1
                        row["name"] = "not_exists"
                        res = get_query_id(ctx, o_bones, row)
                        sts = self.Z.test_result(z0ctx, msg, 1, len(res))
                    if sts == TEST_SUCCESS:
                        sts = self.Z.test_result(z0ctx, msg, row["id"], res[0])

            for _i in range(2):
                if sts == TEST_SUCCESS:
                    # repeat 2 time to check side effects
                    row["id"] = ""
                    row["name"] = "not_exists"
                    res = get_query_id(ctx, o_bones, row)
                    sts = self.Z.test_result(z0ctx, msg, [], res)
        return sts

    def test_09(self, z0ctx):
        msg = "get_query_id (id)"
        sts = TEST_SUCCESS
        for model in ("res.zero", "res.one", "res.two"):
            if sts == TEST_SUCCESS:
                oerp, ctx, o_model, csv_obj, csv_fn, row = self.init_test(model)
                if model == "res.zero":
                    tres = ID_0
                elif model == "res.one":
                    tres = ID_1
                elif model == "res.two":
                    tres = REF_ID
                row["id"] = tres
                row["name"] = "myname"
                row["other"] = "=${res.zero:myname}"
                res = get_query_id(ctx, o_model, row)
                sts = self.Z.test_result(z0ctx, msg, [tres], res)
        return sts

    def test_10(self, z0ctx):
        msg = "actions"
        oerp, ctx, o_model, csv_obj, csv_fn, row = self.init_test("res.zero")
        ctx = create_act_list(ctx)
        sts = TEST_SUCCESS
        lx = ctx["_lx_act"]
        if isaction(ctx, lx[0]):
            ctx["actions_db"] = lx[0]
            ctx["actions_mc"] = lx[1]
        else:
            sts = self.Z.test_result(z0ctx, msg, True, False)
        if sts == TEST_SUCCESS:
            sts = self.Z.test_result(z0ctx, msg, True, check_4_actions(ctx))
        if sts == TEST_SUCCESS:
            ctx["actions_mc"] = "not_existent_action"
            sts = self.Z.test_result(z0ctx, msg, False, check_4_actions(ctx))
        if sts == TEST_SUCCESS:
            lx.append(MY_ACT)
            ctx["actions_mc"] = "unit_test"
            sts = self.Z.test_result(z0ctx, msg, True, check_4_actions(ctx))
        if sts == TEST_SUCCESS:
            sts = do_single_action(ctx, "unit_test")
            sts = self.Z.test_result(z0ctx, msg, TEST_SUCCESS, sts)
        if sts == TEST_SUCCESS:
            sts = do_single_action(ctx, MY_ACT)
            sts = self.Z.test_result(z0ctx, msg, TEST_SUCCESS, sts)
        return sts


#
# Run main if executed as a script
if __name__ == "__main__":
    exit(
        z0test.main_local(
            z0test.parseoptest(sys.argv[1:], version=version()), RegressionTest
        )
    )





# -*- coding: utf-8 -*-
import os.path as pth
import sys

from setuptools import find_packages, setup

name = "clodoo"
github_url = "https://github.com/zeroincombenze/tools"
author = "Antonio Maria Vigliotti"
author_email = "antoniomaria.vigliotti@gmail.com"
source_url = "%s/tree/master/%s" % (github_url, name)
doc_url = "https://zeroincombenze-tools.readthedocs.io/en/latest/#clodoo"
changelog_url = "%s/blob/master/%s/egg-info/CHANGELOG.rst" % (github_url, name)
try:
    long_description = open(pth.join(pth.dirname(__file__), "README.rst")).read()
except IOError:
    long_description = ""

if sys.version_info >= (3, 0):
    install_requires = [
        "future",
        "jsonlib-python3",
        "openpyxl",
        "odoorpc<0.10.0",
        "oerplib3",
        "odoo-client-lib",
        "os0",
        "psycopg2-binary",
        "python-plus",
        "unidecode",
        "z0lib",
    ]
else:
    install_requires = [
        "future",
        "jsonlib",
        "openpyxl<=3.0",
        "odoorpc<0.10.0",
        "oerplib",
        "os0",
        "psycopg2-binary",
        "python-plus",
        "unidecode==1.2.0",
        "z0lib",
    ]

setup(
    name=name,
    version="2.0.9",
    description="Do massive operations on Odoo Cloud",
    long_description=long_description,
    classifiers=[
        "Development Status :: 4 - Beta",
        "License :: OSI Approved :: GNU Affero General Public License v3",
        "Operating System :: POSIX",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Intended Audience :: Developers",
        "Topic :: Software Development",
        "Topic :: Software Development :: Libraries",
    ],
    keywords="odoo",
    url=github_url,
    project_urls={
        "Documentation": doc_url,
        "Source": source_url,
        "Changelog": changelog_url,
    },
    author=author,
    author_email=author_email,
    license="Affero GPL",
    install_requires=install_requires,
    packages=find_packages(exclude=["docs", "examples", "tests", "egg-info", "junk"]),
    package_data={
        "": [
            "scripts/setup.info",
            "./manage_db",
            "./manage_odoo",
            "./manage_odoo.man",
            "./odoo_install_repository",
            "./odoorc",
            "./transodoo.xlsx",
            "./bck_filestore.sh",
        ]
    },
    entry_points={
        "console_scripts": [
            "clodoo-info = clodoo.scripts.main:main",
            # "clodoo.py = clodoo.clodoo_main:main",
            "transodoo.py = clodoo.transodoo:main",
        ]
    },
    zip_safe=False,
)





#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import sys
import csv

from os0 import os0

try:
    from clodoo import clodoo
except ImportError:
    import clodoo
try:
    from z0lib.z0lib import z0lib
except ImportError:
    try:
        from z0lib import z0lib
    except ImportError:
        import z0lib
# import transodoo
# import pdb

__version__ = "2.0.9"


VERSIONS = ["vg7", "61", "70", "80", "90", "100", "110", "120"]
VERSIONS_PLUS = ["vg7", "61", "70", "80", "90", "100", "110", "120", "0"]
ALIAS = {}


def get_summary(info):
    infos = info.split("\n")
    res = ""
    for i in range(len(infos)):
        if infos[i]:
            res = infos[i]
            break
    return res


def new_env():
    mod2xtl = {}
    for id in VERSIONS:
        mod2xtl[id] = []
    return mod2xtl


def add_elem(mod2xtl, id, elem):
    try:
        mod2xtl[id].append(elem)
    except BaseException:
        pass


def sort_data(mod2xtl):
    for id in VERSIONS:
        sorted_list = []
        for item in sorted(mod2xtl[id]):
            if item:
                sorted_list.append(item)
        mod2xtl[id] = sorted_list
        idx = "ix" + id
        mod2xtl[idx] = 0
    mod2xtl["0"] = sorted(infos.keys())
    mod2xtl["ix0"] = 0


def get_next(mod2xtl):
    item = "~"
    for id in VERSIONS_PLUS:
        idx = "ix" + id
        ix = mod2xtl[idx]
        if ix < len(mod2xtl[id]):
            itm = mod2xtl[id][ix]
            if itm < item:
                item = itm
    vers = []
    for id in VERSIONS_PLUS:
        idx = "ix" + id
        ix = mod2xtl[idx]
        if ix < len(mod2xtl[id]):
            itm = mod2xtl[id][ix]
            if itm == item:
                vers.append(id)
                mod2xtl[idx] += 1
    return item, vers


def get_realname(item):
    global ALIAS
    if item.strip() in ALIAS:
        return ALIAS[item.strip()]
    return item.strip()


parser = z0lib.parseoptargs(
    "Modules to install", "© 2017-2020 by SHS-AV s.r.l.", version=__version__
)
parser.add_argument("-h")
parser.add_argument(
    "-c",
    "--config",
    help="configuration command file",
    dest="conf_fn",
    metavar="file",
    default="./clodoo.conf",
)
parser.add_argument(
    "-d",
    "--dbname",
    help="DB name to connect",
    dest="db_name",
    metavar="file",
    default="",
)
parser.add_argument("-n")
parser.add_argument("-q")
parser.add_argument("-V")
parser.add_argument("-v")
ctx = parser.parseoptargs(sys.argv[1:], apply_conf=False)
# pdb.set_trace()
mod2xtl = new_env()
infos = {}
if os.path.isfile("Odoo_moduli.csv"):
    with open("Odoo_moduli.csv", "rb") as f:
        hdr = False
        reader = csv.reader(f)
        for row in reader:
            if not hdr:
                hdr = True
                continue
            # infos[tech_name] = {description, author, notes, vers, repos, oev)
            item = row[1]
            infos[item] = [
                os0.b(row[0]),
                os0.b(row[2]),
                os0.b(row[4]),
                os0.b(row[3]),
                os0.b(row[5]),
                "",
            ]
            add_elem(mod2xtl, "0", item)

if ctx["db_name"]:
    uid, ctx = clodoo.oerp_set_env(confn=ctx["conf_fn"], db=ctx["db_name"], ctx=ctx)
    ixver = ctx["oe_version"].split(".")[0] + ctx["oe_version"].split(".")[1]
    model = "ir.module.module"
    ids = clodoo.searchL8(ctx, model, [("state", "=", "installed")])
    for id in ids:
        try:
            module = clodoo.browseL8(ctx, model, id)
            item = module.name
            if item in infos:
                notes = infos[item][2]
                summary = get_summary(infos[item][0])
            else:
                notes = ""
                summary = get_summary(os0.b(module.summary))
            infos[item] = [
                summary,
                os0.b(module.author),
                notes,
                os0.b(module.installed_version),
                "",
                ixver,
            ]
        except BaseException:
            pass
with open("moduli_alias.csv", "rb") as f:
    lines = f.read().split("\n")
    for line in lines:
        pv = line.split("=")
        if len(pv) == 2 and pv[0] not in ALIAS:
            ALIAS[pv[0].strip()] = os0.b(pv[1].strip())

with open("moduli_da_installare_vg7.csv", "rb") as f:
    hdr = False
    reader = csv.reader(f)
    for row in reader:
        if not hdr:
            hdr = True
            continue
        # print row[1]
        add_elem(mod2xtl, "vg7", get_realname(row[1]))

with open("code/z0_install_10.conf", "rb") as f:
    lines = f.read().split("\n")
    for line in lines:
        if line.startswith("install_modules_"):
            pv = line.split("=")
            prm = pv[0]
            ver = prm[16:]
            id = ver.split(".")[0] + ver.split(".")[1]
            rows = pv[1].split(",")
            for row in rows:
                add_elem(mod2xtl, id, get_realname(row))

sort_data(mod2xtl)
print(",".join(mod2xtl["80"]))
item = ""
ctrs = {}
for id in VERSIONS_PLUS:
    ctrs[id] = 0
# import pdb
# pdb.set_trace()
fd = open("moduli_da_installare.csv", "w")
fmto = (
    "%-40.40s %-3.3s %-3.3s %-3.3s %-3.3s %-3.3s"
    " %-4.4s %-4.4s %-4.4s %-40.40s %-20.20s %-10.10s"
)
fmtx = '"%s",%s,%s,%s,%s,%s,%s,%s,%s,"%s","%s","%s"\n'
print(
    fmto
    % (
        "Technical Name",
        VERSIONS[0],
        "%s.%s" % (VERSIONS[1][0:-1], VERSIONS[1][-1]),
        "%s.%s" % (VERSIONS[2][0:-1], VERSIONS[2][-1]),
        "%s.%s" % (VERSIONS[3][0:-1], VERSIONS[3][-1]),
        "%s.%s" % (VERSIONS[4][0:-1], VERSIONS[4][-1]),
        "%s.%s" % (VERSIONS[5][0:-1], VERSIONS[5][-1]),
        "%s.%s" % (VERSIONS[6][0:-1], VERSIONS[6][-1]),
        "%s.%s" % (VERSIONS[7][0:-1], VERSIONS[7][-1]),
        "Description",
        "Author",
        "Notes",
    )
)
line = fmtx % (
    "Technical Name",
    VERSIONS[0],
    "%s.%s" % (VERSIONS[1][0:-1], VERSIONS[1][-1]),
    "%s.%s" % (VERSIONS[2][0:-1], VERSIONS[2][-1]),
    "%s.%s" % (VERSIONS[3][0:-1], VERSIONS[3][-1]),
    "%s.%s" % (VERSIONS[4][0:-1], VERSIONS[4][-1]),
    "%s.%s" % (VERSIONS[5][0:-1], VERSIONS[5][-1]),
    "%s.%s" % (VERSIONS[6][0:-1], VERSIONS[6][-1]),
    "%s.%s" % (VERSIONS[7][0:-1], VERSIONS[7][-1]),
    "Description",
    "Author",
    "Notes",
)
fd.write(line)
while item != "~":
    item, vers = get_next(mod2xtl)
    if item != "~":
        datas = []
        for id in VERSIONS:
            if id in vers:
                datas.append("OK")
                ctrs[id] += 1
            else:
                datas.append("x")
        if item in infos:
            des = infos[item][0]
            author = infos[item][1]
            note = infos[item][2]
        else:
            des = author = note = ""
        # if infos[item][5]:
        #     for i,v in enumerate(VERSIONS):
        #         if v == infos[item][5]:
        #             datas[i] = 'OK'
        try:
            print(
                fmto
                % (
                    item,
                    datas[0],
                    datas[1],
                    datas[2],
                    datas[3],
                    datas[4],
                    datas[5],
                    datas[6],
                    datas[7],
                    des,
                    author,
                    note,
                )
            )
        except BaseException:
            print(
                fmto
                % (
                    item,
                    datas[0],
                    datas[1],
                    datas[2],
                    datas[3],
                    datas[4],
                    datas[5],
                    datas[6],
                    datas[7],
                    des,
                    "",
                    note,
                )
            )
        try:
            line = fmtx % (
                item,
                datas[0],
                datas[1],
                datas[2],
                datas[3],
                datas[4],
                datas[5],
                datas[6],
                datas[7],
                des,
                author,
                note,
            )
        except BaseException:
            line = fmtx % (
                item,
                datas[0],
                datas[1],
                datas[2],
                datas[3],
                datas[4],
                datas[5],
                datas[6],
                datas[7],
                des,
                "",
                note,
            )
        fd.write(line)
line = "%-65.65s" % "TOTALE"
for id in VERSIONS:
    line = "%s %3s" % (line, ctrs[id])
print(line)
line = "TOTALE"
for id in VERSIONS:
    line = "%s,%s" % (line, ctrs[id])
fd.write(line)
fd.close()


